﻿using System;
using Newtonsoft.Json;
using System.Collections.Generic;
using WalletConnectSharp.Unity;
using WalletConnectSharp.Unity.Models;

namespace Assets.Scripts.WalletConnectSharp.Unity.UI
{
    internal class SupportedWalletList
    {
        class WalletCompare : IComparer<WalletSelectItem>
        {
            public int Compare(WalletSelectItem x, WalletSelectItem y)
            {
                return x.Name.ToLower().CompareTo(y.Name.ToLower());
            }
        }

        public static List<WalletSelectItem> SupportedWalletNames()
        {
            List<WalletSelectItem> result = new List<WalletSelectItem>();
            string json = WalletListJson();
            Dictionary<string, AppEntry> data = JsonConvert.DeserializeObject<Dictionary<string, AppEntry>>(json);

            foreach (var id in data.Keys)
            {
                if (data[id].app != null && data[id].app.ios != null && data[id].app.ios.Length > 0)
                {
                    result.Add(new WalletSelectItem(id, data[id].name));
                }
                
            }

            result.Sort(new WalletCompare());

            return result;
        }

        public static string WalletListJson()
        {
            string wallets = "{<Q>1ae92b26df02f0abca6304df07debccd18262fdf5fe82daa81593582dac9a369<Q>:{<Q>id<Q>:<Q>1ae92b26df02f0abca6304df07debccd18262fdf5fe82daa81593582dac9a369<Q>,<Q>name<Q>:<Q>Rainbow<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://rainbow.me/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>2cc2f20c-840b-497a-c028-dbb481d49700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2cc2f20c-840b-497a-c028-dbb481d49700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2cc2f20c-840b-497a-c028-dbb481d49700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2cc2f20c-840b-497a-c028-dbb481d49700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/rainbow-ethereum-wallet/id1457119021<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>rainbow:<Q>,<Q>universal<Q>:<Q>https://rnbwapp.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Rainbow<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#001e59<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>4622a2b2d6af1c9844944291e5e7351a6aa24cd7b23099efac1b2fd875da31a0<Q>:{<Q>id<Q>:<Q>4622a2b2d6af1c9844944291e5e7351a6aa24cd7b23099efac1b2fd875da31a0<Q>,<Q>name<Q>:<Q>Trust Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://trustwallet.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>0528ee7e-16d1-4089-21e3-bbfb41933100<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0528ee7e-16d1-4089-21e3-bbfb41933100/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0528ee7e-16d1-4089-21e3-bbfb41933100/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0528ee7e-16d1-4089-21e3-bbfb41933100/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/apple-store/id1288339409<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.wallet.crypto.trustapp<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>trust:<Q>,<Q>universal<Q>:<Q>https://link.trustwallet.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Trust<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#3375bb<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>cf21952a9bc8108bf13b12c92443751e2cc388d27008be4201b92bbc6d83dd46<Q>:{<Q>id<Q>:<Q>cf21952a9bc8108bf13b12c92443751e2cc388d27008be4201b92bbc6d83dd46<Q>,<Q>name<Q>:<Q>Argent<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://argent.xyz/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>ce5fbfe8-13b5-4f5f-184a-34f6ee7a3d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ce5fbfe8-13b5-4f5f-184a-34f6ee7a3d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ce5fbfe8-13b5-4f5f-184a-34f6ee7a3d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ce5fbfe8-13b5-4f5f-184a-34f6ee7a3d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/argent/id1358741926<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=im.argent.contractwalletclient<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>argent://app<Q>,<Q>universal<Q>:<Q>https://argent.link/app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Argent<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ff875b<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>c57ca95b47569778a828d19178114f4db188b89b763c899ba0be274e97267d96<Q>:{<Q>id<Q>:<Q>c57ca95b47569778a828d19178114f4db188b89b763c899ba0be274e97267d96<Q>,<Q>name<Q>:<Q>MetaMask<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://metamask.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>fb5269f0-1870-42d6-82b4-26d27626e800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/fb5269f0-1870-42d6-82b4-26d27626e800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/fb5269f0-1870-42d6-82b4-26d27626e800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/fb5269f0-1870-42d6-82b4-26d27626e800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/metamask/id1438144202<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.metamask<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>metamask:<Q>,<Q>universal<Q>:<Q>https://metamask.app.link<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>MetaMask<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>225affb176778569276e484e1b92637ad061b01e13a048b35a9d280c3b58970f<Q>:{<Q>id<Q>:<Q>225affb176778569276e484e1b92637ad061b01e13a048b35a9d280c3b58970f<Q>,<Q>name<Q>:<Q>Gnosis Safe Multisig<Q>,<Q>description<Q>:null,<Q>homepage<Q>:<Q>https://gnosis-safe.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>0b7e0f05-0a5b-4f3c-315d-59c1c4c22c00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0b7e0f05-0a5b-4f3c-315d-59c1c4c22c00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0b7e0f05-0a5b-4f3c-315d-59c1c4c22c00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0b7e0f05-0a5b-4f3c-315d-59c1c4c22c00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://gnosis-safe.io/app/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/id1515759131<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.gnosis.safe<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>gnosissafe:<Q>,<Q>universal<Q>:<Q>https://gnosis-safe.io/<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Gnosis Safe<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#091828<Q>,<Q>secondary<Q>:<Q>#fff<Q>}}},<Q>f2436c67184f158d1beda5df53298ee84abfc367581e4505134b5bcf5f46697d<Q>:{<Q>id<Q>:<Q>f2436c67184f158d1beda5df53298ee84abfc367581e4505134b5bcf5f46697d<Q>,<Q>name<Q>:<Q>Crypto.com | DeFi Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://crypto.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7c5ff577-a68d-49c5-02cd-3d83637b0b00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7c5ff577-a68d-49c5-02cd-3d83637b0b00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7c5ff577-a68d-49c5-02cd-3d83637b0b00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7c5ff577-a68d-49c5-02cd-3d83637b0b00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/id1262148500<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=co.mona.android<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>cryptowallet:<Q>,<Q>universal<Q>:<Q>https://wallet.crypto.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Crypto.com<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1199fa<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>0b58bf037bf943e934706796fb017d59eace1dadcbc1d9fe24d9b46629e5985c<Q>:{<Q>id<Q>:<Q>0b58bf037bf943e934706796fb017d59eace1dadcbc1d9fe24d9b46629e5985c<Q>,<Q>name<Q>:<Q>Pillar<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://pillarproject.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>0eb8230d-ce4c-42fb-2a57-a84a6eb7ea00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0eb8230d-ce4c-42fb-2a57-a84a6eb7ea00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0eb8230d-ce4c-42fb-2a57-a84a6eb7ea00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0eb8230d-ce4c-42fb-2a57-a84a6eb7ea00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/apple-store/id1346582238<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.pillarproject.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>pillarwallet:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Pillar<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>9d373b43ad4d2cf190fb1a774ec964a1addf406d6fd24af94ab7596e58c291b2<Q>:{<Q>id<Q>:<Q>9d373b43ad4d2cf190fb1a774ec964a1addf406d6fd24af94ab7596e58c291b2<Q>,<Q>name<Q>:<Q>imToken<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://token.im/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>1991f85d-43d4-4165-3502-cd6ef8312b00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1991f85d-43d4-4165-3502-cd6ef8312b00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1991f85d-43d4-4165-3502-cd6ef8312b00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1991f85d-43d4-4165-3502-cd6ef8312b00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/us/app/imtoken2/id1384798940<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=im.token.app<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>imtokenv2:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>imToken<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>dceb063851b1833cbb209e3717a0a0b06bf3fb500fe9db8cd3a553e4b1d02137<Q>:{<Q>id<Q>:<Q>dceb063851b1833cbb209e3717a0a0b06bf3fb500fe9db8cd3a553e4b1d02137<Q>,<Q>name<Q>:<Q>ONTO<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://onto.app/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d22b2a4b-5562-49ba-506b-6d5986914600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d22b2a4b-5562-49ba-506b-6d5986914600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d22b2a4b-5562-49ba-506b-6d5986914600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d22b2a4b-5562-49ba-506b-6d5986914600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/onto-an-ontology-dapp/id1436009823<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.github.ontio.onto<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>ontoprovider:<Q>,<Q>universal<Q>:<Q>https://onto.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>ONTO<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>20459438007b75f4f4acb98bf29aa3b800550309646d375da5fd4aac6c2a2c66<Q>:{<Q>id<Q>:<Q>20459438007b75f4f4acb98bf29aa3b800550309646d375da5fd4aac6c2a2c66<Q>,<Q>name<Q>:<Q>TokenPocket<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://tokenpocket.pro/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d436aaa8-28c7-4efb-5df4-3e1af8012c00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d436aaa8-28c7-4efb-5df4-3e1af8012c00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d436aaa8-28c7-4efb-5df4-3e1af8012c00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d436aaa8-28c7-4efb-5df4-3e1af8012c00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/tokenpocket-trusted-wallet/id1436028697<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=vip.mytokenpocket<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>tpoutside:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>TokenPocket<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#2982fe<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>7674bb4e353bf52886768a3ddc2a4562ce2f4191c80831291218ebd90f5f5e26<Q>:{<Q>id<Q>:<Q>7674bb4e353bf52886768a3ddc2a4562ce2f4191c80831291218ebd90f5f5e26<Q>,<Q>name<Q>:<Q>MathWallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://mathwallet.org/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>26a8f588-3231-4411-60ce-5bb6b805a700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/26a8f588-3231-4411-60ce-5bb6b805a700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/26a8f588-3231-4411-60ce-5bb6b805a700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/26a8f588-3231-4411-60ce-5bb6b805a700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/math-wallet-blockchain-wallet/id1383637331<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.medishares.android<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>mathwallet:<Q>,<Q>universal<Q>:<Q>https://www.mathwallet.org<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>MathWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>afbd95522f4041c71dd4f1a065f971fd32372865b416f95a0b1db759ae33f2a7<Q>:{<Q>id<Q>:<Q>afbd95522f4041c71dd4f1a065f971fd32372865b416f95a0b1db759ae33f2a7<Q>,<Q>name<Q>:<Q>Steakwallet<Q>,<Q>description<Q>:<Q>Multi chain, self custodial DeFi wallet<Q>,<Q>homepage<Q>:<Q>https://steakwallet.fi<Q>,<Q>chains<Q>:[<Q>cosmos:cosmoshub-4<Q>,<Q>cosmos:kava-4<Q>,<Q>cosmos:columbus-4<Q>,<Q>eip155:1<Q>,<Q>eip155:10<Q>,<Q>eip155:56<Q>,<Q>eip155:100<Q>,<Q>eip155:137<Q>,<Q>eip155:1284<Q>,<Q>eip155:1285<Q>,<Q>eip155:42161<Q>,<Q>eip155:42220<Q>,<Q>eip155:43114<Q>,<Q>eip155:1666600000<Q>],<Q>versions<Q>:[<Q>1<Q>,<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>700457dc-d772-46f3-7332-8c80a3dcb800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/700457dc-d772-46f3-7332-8c80a3dcb800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/700457dc-d772-46f3-7332-8c80a3dcb800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/700457dc-d772-46f3-7332-8c80a3dcb800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/de/app/steakwallet/id1569375204?l=en<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=fi.steakwallet.app<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>steakwallet<Q>,<Q>universal<Q>:<Q>https://links.steakwallet.fi<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Steakwallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#FF3B9A<Q>,<Q>secondary<Q>:<Q>#000000<Q>}}},<Q>ccb714920401f7d008dbe11281ae70e3a4bfb621763b187b9e4a3ce1ab8faa3b<Q>:{<Q>id<Q>:<Q>ccb714920401f7d008dbe11281ae70e3a4bfb621763b187b9e4a3ce1ab8faa3b<Q>,<Q>name<Q>:<Q>BitPay<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://bitpay.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>97d4429f-eaf0-4302-87f5-9d26d46fe700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/97d4429f-eaf0-4302-87f5-9d26d46fe700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/97d4429f-eaf0-4302-87f5-9d26d46fe700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/97d4429f-eaf0-4302-87f5-9d26d46fe700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/bitpay-buy-crypto/id1149581638<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.bitpay.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>bitpay:<Q>,<Q>universal<Q>:<Q>https://link.bitpay.com/wallet<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>BitPay<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1a3c8b<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>83f26999937cbc2e2014655796da4b05f77c1de9413a0ee6d0c6178ebcfc3168<Q>:{<Q>id<Q>:<Q>83f26999937cbc2e2014655796da4b05f77c1de9413a0ee6d0c6178ebcfc3168<Q>,<Q>name<Q>:<Q>WallETH<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://walleth.org/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:100<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:5<Q>,<Q>eip155:10<Q>,<Q>eip155:77<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>8b59dafd-9150-46be-9793-34e6d3298100<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8b59dafd-9150-46be-9793-34e6d3298100/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8b59dafd-9150-46be-9793-34e6d3298100/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8b59dafd-9150-46be-9793-34e6d3298100/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=org.walleth<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>WallETH<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>19177a98252e07ddfc9af2083ba8e07ef627cb6103467ffebb3f8f4205fd7927<Q>:{<Q>id<Q>:<Q>19177a98252e07ddfc9af2083ba8e07ef627cb6103467ffebb3f8f4205fd7927<Q>,<Q>name<Q>:<Q>Ledger Live<Q>,<Q>description<Q>:<Q>Web3 Wallet from the company that produced the world's most secure crypto hardware device.<Q>,<Q>homepage<Q>:<Q>https://www.ledger.com/ledger-live<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:4<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a7f416de-aa03-4c5e-3280-ab49269aef00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a7f416de-aa03-4c5e-3280-ab49269aef00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a7f416de-aa03-4c5e-3280-ab49269aef00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a7f416de-aa03-4c5e-3280-ab49269aef00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.ledger.com/ledger-live/download<Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/app/id1361671700<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.ledger.live<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>ledgerlive:<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:<Q>ledgerlive:<Q>,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Ledger<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000000<Q>,<Q>secondary<Q>:<Q>#FF5300<Q>}}},<Q>71dad538ba02a9b321041d388f9c1efe14e0d1915a2ea80a90405d2f6b67a33d<Q>:{<Q>id<Q>:<Q>71dad538ba02a9b321041d388f9c1efe14e0d1915a2ea80a90405d2f6b67a33d<Q>,<Q>name<Q>:<Q>Authereum<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://authereum.org<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>deb6ec25-fcec-4b1b-c536-df3b4fb92b00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/deb6ec25-fcec-4b1b-c536-df3b4fb92b00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/deb6ec25-fcec-4b1b-c536-df3b4fb92b00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/deb6ec25-fcec-4b1b-c536-df3b4fb92b00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://authereum.com/i/account<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Authereum<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>138f51c8d00ac7b9ac9d8dc75344d096a7dfe370a568aa167eabc0a21830ed98<Q>:{<Q>id<Q>:<Q>138f51c8d00ac7b9ac9d8dc75344d096a7dfe370a568aa167eabc0a21830ed98<Q>,<Q>name<Q>:<Q>AlphaWallet<Q>,<Q>description<Q>:<Q>Ethereum, EVM wallet<Q>,<Q>homepage<Q>:<Q>https://alphawallet.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:5<Q>,<Q>eip155:10<Q>,<Q>eip155:42<Q>,<Q>eip155:56<Q>,<Q>eip155:61<Q>,<Q>eip155:69<Q>,<Q>eip155:77<Q>,<Q>eip155:97<Q>,<Q>eip155:99<Q>,<Q>eip155:100<Q>,<Q>eip155:137<Q>,<Q>eip155:200<Q>,<Q>eip155:256<Q>,<Q>eip155:338<Q>,<Q>eip155:4002<Q>,<Q>eip155:42161<Q>,<Q>eip155:43113<Q>,<Q>eip155:43114<Q>,<Q>eip155:11297108099<Q>,<Q>eip155:11297108109<Q>],<Q>versions<Q>:[<Q>1<Q>,<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>1746957d-bbfd-45aa-0695-37f27b67c200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1746957d-bbfd-45aa-0695-37f27b67c200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1746957d-bbfd-45aa-0695-37f27b67c200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1746957d-bbfd-45aa-0695-37f27b67c200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/alphawallet-eth-wallet/id1358230430<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.stormbird.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>awallet:<Q>,<Q>universal<Q>:<Q>https://aw.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>AlphaWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:null}}},<Q>2863183c3299d820fb9a4cb8aab4a34f50380c9992e8be871fd60a62e8d36481<Q>:{<Q>id<Q>:<Q>2863183c3299d820fb9a4cb8aab4a34f50380c9992e8be871fd60a62e8d36481<Q>,<Q>name<Q>:<Q>1inch Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://1inch.io/wallet/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>dce1ee99-403f-44a9-9f94-20de30616500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dce1ee99-403f-44a9-9f94-20de30616500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dce1ee99-403f-44a9-9f94-20de30616500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dce1ee99-403f-44a9-9f94-20de30616500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/1inch-defi-wallet/id1546049391<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>1inch:<Q>,<Q>universal<Q>:<Q>https://wallet.1inch.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>1inch<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1f242e<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>797c615e2c556b610c048eb35535f212c0dd58de5d03e763120e90a7d1350a77<Q>:{<Q>id<Q>:<Q>797c615e2c556b610c048eb35535f212c0dd58de5d03e763120e90a7d1350a77<Q>,<Q>name<Q>:<Q>Huobi Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://huobiwallet.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>5a96d19c-93d3-4e6a-624a-16f52773d000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5a96d19c-93d3-4e6a-624a-16f52773d000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5a96d19c-93d3-4e6a-624a-16f52773d000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5a96d19c-93d3-4e6a-624a-16f52773d000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/id1433883012<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.huobionchainwallet.gp<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>huobiwallet:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Huobi<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#2d67f8<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>efba9ae0a9e0fdd9e3e055ddf3c8e75f294babb8aea3499456eff27f771fda61<Q>:{<Q>id<Q>:<Q>efba9ae0a9e0fdd9e3e055ddf3c8e75f294babb8aea3499456eff27f771fda61<Q>,<Q>name<Q>:<Q>Eidoo<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://eidoo.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>ef5b8bcf-00d5-457d-e161-9911e4788700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ef5b8bcf-00d5-457d-e161-9911e4788700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ef5b8bcf-00d5-457d-e161-9911e4788700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ef5b8bcf-00d5-457d-e161-9911e4788700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/app/apple-store/id1279896253<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.eidoo.wallet.prodnet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>eidoo:<Q>,<Q>universal<Q>:<Q>https://eidoo.io/crypto-wallet<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Eidoo<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#37b39e<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>61f6e716826ae8455ad16abc5ec31e4fd5d6d2675f0ce2dee3336335431f720e<Q>:{<Q>id<Q>:<Q>61f6e716826ae8455ad16abc5ec31e4fd5d6d2675f0ce2dee3336335431f720e<Q>,<Q>name<Q>:<Q>MYKEY<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://mykey.org<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>df0306ae-f33c-454f-8679-39aed351c800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/df0306ae-f33c-454f-8679-39aed351c800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/df0306ae-f33c-454f-8679-39aed351c800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/df0306ae-f33c-454f-8679-39aed351c800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/app/id1458318224<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.mykey.id<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>mykeywalletconnect:<Q>,<Q>universal<Q>:<Q>https://mykey.org<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>MYKEY<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>c36c969aefd9d2e8ce6c1b90da90c0970699aee14b07c7a84b1124873d84c53d<Q>:{<Q>id<Q>:<Q>c36c969aefd9d2e8ce6c1b90da90c0970699aee14b07c7a84b1124873d84c53d<Q>,<Q>name<Q>:<Q>Loopring Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://loopring.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>2972bbf8-0891-414a-f63c-8d3bcf661d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2972bbf8-0891-414a-f63c-8d3bcf661d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2972bbf8-0891-414a-f63c-8d3bcf661d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2972bbf8-0891-414a-f63c-8d3bcf661d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/loopring-smart-wallet/id1550921126<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=loopring.defi.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Loopring<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>6bb4596640ce9f8c02fbaa83e3685425455a0917d025608b4abc53bfe55887c6<Q>:{<Q>id<Q>:<Q>6bb4596640ce9f8c02fbaa83e3685425455a0917d025608b4abc53bfe55887c6<Q>,<Q>name<Q>:<Q>TrustVault<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://trustology.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>fc5e0354-cc1e-490d-fb62-477e83148000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/fc5e0354-cc1e-490d-fb62-477e83148000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/fc5e0354-cc1e-490d-fb62-477e83148000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/fc5e0354-cc1e-490d-fb62-477e83148000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/apple-store/id1455959680<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>TrustVault<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>185850e869e40f4e6c59b5b3f60b7e63a72e88b09e2a43a40b1fd0f237e49e9a<Q>:{<Q>id<Q>:<Q>185850e869e40f4e6c59b5b3f60b7e63a72e88b09e2a43a40b1fd0f237e49e9a<Q>,<Q>name<Q>:<Q>Atomic<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://atomicwallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>adb1ed3f-722c-48a0-441f-c75038a9a300<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/adb1ed3f-722c-48a0-441f-c75038a9a300/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/adb1ed3f-722c-48a0-441f-c75038a9a300/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/adb1ed3f-722c-48a0-441f-c75038a9a300/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/atomic-wallet/id1478257827<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.atomicwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Atomic<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>b021913ba555948a1c81eb3d89b372be46f8354e926679de648e4fa2938bed3e<Q>:{<Q>id<Q>:<Q>b021913ba555948a1c81eb3d89b372be46f8354e926679de648e4fa2938bed3e<Q>,<Q>name<Q>:<Q>Coin98<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://coin98.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>dee547be-936a-4c92-9e3f-7a2350a62e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dee547be-936a-4c92-9e3f-7a2350a62e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dee547be-936a-4c92-9e3f-7a2350a62e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dee547be-936a-4c92-9e3f-7a2350a62e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://ios.coin98.com/<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=coin98.crypto.finance.media<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>coin98:<Q>,<Q>universal<Q>:<Q>https://coin98.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Coin98<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#272727<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>1f69170bf7a9bdcf89403ec012659b7124e158f925cdd4a2be49274c24cf5e5d<Q>:{<Q>id<Q>:<Q>1f69170bf7a9bdcf89403ec012659b7124e158f925cdd4a2be49274c24cf5e5d<Q>,<Q>name<Q>:<Q>CoolWallet<Q>,<Q>description<Q>:<Q>Building secure and user-friendly blockchain solutions<Q>,<Q>homepage<Q>:<Q>https://coolwallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e97c4a6d-cd91-4331-ac98-196141df5300<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e97c4a6d-cd91-4331-ac98-196141df5300/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e97c4a6d-cd91-4331-ac98-196141df5300/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e97c4a6d-cd91-4331-ac98-196141df5300/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://itunes.apple.com/us/app/coolwallet-s-2018/id1328764142<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.coolbitx.cwsapp<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>coolwallet:<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>CoolWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>beea4e71c2ffbb48b59b21e33fb0049ef6522585aa9c8a33a97d3e1c81f16693<Q>:{<Q>id<Q>:<Q>beea4e71c2ffbb48b59b21e33fb0049ef6522585aa9c8a33a97d3e1c81f16693<Q>,<Q>name<Q>:<Q>Alice<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://alicedapp.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>bd1ce165-9b3a-4925-73c1-b329ca13e900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bd1ce165-9b3a-4925-73c1-b329ca13e900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bd1ce165-9b3a-4925-73c1-b329ca13e900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bd1ce165-9b3a-4925-73c1-b329ca13e900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/au/app/alice-browser/id1472818028<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Alice<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>468b4ab3582757233017ec10735863489104515ab160c053074905a1eecb7e63<Q>:{<Q>id<Q>:<Q>468b4ab3582757233017ec10735863489104515ab160c053074905a1eecb7e63<Q>,<Q>name<Q>:<Q>D'CENT Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://dcentwallet.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:30<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>1efb49ec-2bab-4fa1-f2f2-4392c64ed000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1efb49ec-2bab-4fa1-f2f2-4392c64ed000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1efb49ec-2bab-4fa1-f2f2-4392c64ed000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1efb49ec-2bab-4fa1-f2f2-4392c64ed000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/dcent-hardware-wallet/id1447206611<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.kr.iotrust.dcent.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>dcent:<Q>,<Q>universal<Q>:<Q>https://link.dcentwallet.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>D'CENT<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#fff<Q>,<Q>secondary<Q>:<Q>#fff<Q>}}},<Q>29f4a70ad5993f3f73ae8119f0e78ecbae51deec2a021a770225c644935c0f09<Q>:{<Q>id<Q>:<Q>29f4a70ad5993f3f73ae8119f0e78ecbae51deec2a021a770225c644935c0f09<Q>,<Q>name<Q>:<Q>ZelCore<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://zel.network<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>97d45a92-a1f0-46da-95a6-ad5db99f3500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/97d45a92-a1f0-46da-95a6-ad5db99f3500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/97d45a92-a1f0-46da-95a6-ad5db99f3500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/97d45a92-a1f0-46da-95a6-ad5db99f3500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/zelcore/id1436296839<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.zelcash.zelcore<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>zel:<Q>,<Q>universal<Q>:<Q>https://link.zel.network<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>ZelCore<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#232220<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>8240fb8a7b117aed27f04aa8870c714eeb910f7c1b16c9b868e793c1836335b8<Q>:{<Q>id<Q>:<Q>8240fb8a7b117aed27f04aa8870c714eeb910f7c1b16c9b868e793c1836335b8<Q>,<Q>name<Q>:<Q>Nash<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://nash.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>12f981b1-bb0a-4115-009f-317255979600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/12f981b1-bb0a-4115-009f-317255979600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/12f981b1-bb0a-4115-009f-317255979600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/12f981b1-bb0a-4115-009f-317255979600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/nash-app/id1475759236<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.nash.app<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>nash:<Q>,<Q>universal<Q>:<Q>https://nash.io/walletconnect<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Nash<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#0052f3<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>15d7610042217f691385d20e640869dc7273e991b04e8f476417cdc5ec856955<Q>:{<Q>id<Q>:<Q>15d7610042217f691385d20e640869dc7273e991b04e8f476417cdc5ec856955<Q>,<Q>name<Q>:<Q>Coinomi<Q>,<Q>description<Q>:<Q>The blockchain wallet trusted by millions. Securely store, manage, and exchange Bitcoin, Ethereum, and more than 1,770 Tokens and Altcoins.<Q>,<Q>homepage<Q>:<Q>https://www.coinomi.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>3b446d16-a908-40c8-5835-9a6efe90dd00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3b446d16-a908-40c8-5835-9a6efe90dd00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3b446d16-a908-40c8-5835-9a6efe90dd00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3b446d16-a908-40c8-5835-9a6efe90dd00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://itunes.apple.com/app/coinomi-wallet/id1333588809<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.coinomi.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>coinomi:<Q>,<Q>universal<Q>:<Q>https://coinomi.page.link<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Coinomi<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>6ec1ffc9627c3b9f87676da3f7b5796828a6c016d3253e51e771e6f951cb5702<Q>:{<Q>id<Q>:<Q>6ec1ffc9627c3b9f87676da3f7b5796828a6c016d3253e51e771e6f951cb5702<Q>,<Q>name<Q>:<Q>GridPlus<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://gridplus.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>3c928cbd-39dc-4090-c372-d4dcb3c89500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3c928cbd-39dc-4090-c372-d4dcb3c89500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3c928cbd-39dc-4090-c372-d4dcb3c89500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3c928cbd-39dc-4090-c372-d4dcb3c89500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>GridPlus<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>a395dbfc92b5519cbd1cc6937a4e79830187daaeb2c6fcdf9b9cce4255f2dcd5<Q>:{<Q>id<Q>:<Q>a395dbfc92b5519cbd1cc6937a4e79830187daaeb2c6fcdf9b9cce4255f2dcd5<Q>,<Q>name<Q>:<Q>CYBAVO Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://cybavo.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>3117d3ce-b973-4cfd-8fb5-f5d72ed3c200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3117d3ce-b973-4cfd-8fb5-f5d72ed3c200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3117d3ce-b973-4cfd-8fb5-f5d72ed3c200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3117d3ce-b973-4cfd-8fb5-f5d72ed3c200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/tw/app/cybavo-wallet/id1510697681<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.cybavo.btc.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>cybavowallet:<Q>,<Q>universal<Q>:<Q>https://cdn.cybavo.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>CYBAVO<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>c889f5add667a8c69d147d613c7f18a4bd97c2e47c946cabfdd13ec1d596e4a0<Q>:{<Q>id<Q>:<Q>c889f5add667a8c69d147d613c7f18a4bd97c2e47c946cabfdd13ec1d596e4a0<Q>,<Q>name<Q>:<Q>Tokenary<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://tokenary.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:10<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:42161<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>5e481041-dc3c-4a81-373a-76bbde91b800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5e481041-dc3c-4a81-373a-76bbde91b800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5e481041-dc3c-4a81-373a-76bbde91b800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5e481041-dc3c-4a81-373a-76bbde91b800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/tokenary-ethereum-wallet/id1372886601<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q>https://apps.apple.com/us/app/tokenary-ethereum-wallet/id1375542355<Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q>tokenary:<Q>,<Q>universal<Q>:<Q>https://tokenary.io<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Tokenary<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#fff<Q>,<Q>secondary<Q>:<Q>#fff<Q>}}},<Q>3f1bc4a8fd72b3665459ec5c99ee51b424f6beeebe46b45f4a70cf08a84cbc50<Q>:{<Q>id<Q>:<Q>3f1bc4a8fd72b3665459ec5c99ee51b424f6beeebe46b45f4a70cf08a84cbc50<Q>,<Q>name<Q>:<Q>Torus<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://toruswallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>47d03b65-6be7-4004-5dba-7dadef6e6000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/47d03b65-6be7-4004-5dba-7dadef6e6000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/47d03b65-6be7-4004-5dba-7dadef6e6000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/47d03b65-6be7-4004-5dba-7dadef6e6000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://app.tor.us/<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Torus<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>7b83869f03dc3848866e0299bc630aaf3213bea95cd6cecfbe149389cf457a09<Q>:{<Q>id<Q>:<Q>7b83869f03dc3848866e0299bc630aaf3213bea95cd6cecfbe149389cf457a09<Q>,<Q>name<Q>:<Q>Spatium<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://spatium.net/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e85caf96-8e6c-4ac5-5bb3-c13ac7edc700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e85caf96-8e6c-4ac5-5bb3-c13ac7edc700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e85caf96-8e6c-4ac5-5bb3-c13ac7edc700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e85caf96-8e6c-4ac5-5bb3-c13ac7edc700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/spatium/id1404844195<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=capital.spatium.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Spatium<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>0b415a746fb9ee99cce155c2ceca0c6f6061b1dbca2d722b3ba16381d0562150<Q>:{<Q>id<Q>:<Q>0b415a746fb9ee99cce155c2ceca0c6f6061b1dbca2d722b3ba16381d0562150<Q>,<Q>name<Q>:<Q>SafePal<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://safepal.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7c4f63f5-3cf6-4043-3c71-139e6f245000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7c4f63f5-3cf6-4043-3c71-139e6f245000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7c4f63f5-3cf6-4043-3c71-139e6f245000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7c4f63f5-3cf6-4043-3c71-139e6f245000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/safepal-wallet/id1548297139<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.safepal.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>safepalwallet:<Q>,<Q>universal<Q>:<Q>https://link.safepal.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>SafePal<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#0080ff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>d0387325e894a1c4244820260ad7c78bb20d79eeec2fd59ffe3529223f3f84c6<Q>:{<Q>id<Q>:<Q>d0387325e894a1c4244820260ad7c78bb20d79eeec2fd59ffe3529223f3f84c6<Q>,<Q>name<Q>:<Q>Infinito<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://infinitowallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>b07624f3-fb36-45a4-200c-6cb2a930ef00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b07624f3-fb36-45a4-200c-6cb2a930ef00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b07624f3-fb36-45a4-200c-6cb2a930ef00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b07624f3-fb36-45a4-200c-6cb2a930ef00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/infinito-wallet/id1315572736<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.infinito.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Infinito<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>176b83d9268d77438e32aa44770fb37b40d6448740b6a05a97b175323356bd1b<Q>:{<Q>id<Q>:<Q>176b83d9268d77438e32aa44770fb37b40d6448740b6a05a97b175323356bd1b<Q>,<Q>name<Q>:<Q>wallet.io<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://wallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>82cdf116-4355-4e07-88e4-63dc2e253500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/82cdf116-4355-4e07-88e4-63dc2e253500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/82cdf116-4355-4e07-88e4-63dc2e253500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/82cdf116-4355-4e07-88e4-63dc2e253500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/vn/app/wallet-io/id1459857368<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>wallet.io<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>802a2041afdaf4c7e41a2903e98df333c8835897532699ad370f829390c6900f<Q>:{<Q>id<Q>:<Q>802a2041afdaf4c7e41a2903e98df333c8835897532699ad370f829390c6900f<Q>,<Q>name<Q>:<Q>Infinity Wallet<Q>,<Q>description<Q>:<Q>Infinity Wallet is a leading all-in-one one-stop DeFi and Web3 crypto wallet<Q>,<Q>homepage<Q>:<Q>https://infinitywallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:1666600000<Q>],<Q>versions<Q>:[<Q>1<Q>,<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>9f259366-0bcd-4817-0af9-f78773e41900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/9f259366-0bcd-4817-0af9-f78773e41900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/9f259366-0bcd-4817-0af9-f78773e41900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/9f259366-0bcd-4817-0af9-f78773e41900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://infinitywallet.io/<Q>,<Q>ios<Q>:null,<Q>android<Q>:null,<Q>mac<Q>:<Q>https://infinitywallet.io/desktop<Q>,<Q>windows<Q>:<Q>https://infinitywallet.io/desktop<Q>,<Q>linux<Q>:<Q>https://infinitywallet.io/desktop<Q>},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:<Q>infinity:<Q>,<Q>universal<Q>:<Q>https://infinitywallet.io/<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Infinity Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#197fe1<Q>,<Q>secondary<Q>:<Q>#ffffff<Q>}}},<Q>8fb830a15679a8537d84c3852e026a4bdb39d0ee3b387411a91e8f6abafdc1ad<Q>:{<Q>id<Q>:<Q>8fb830a15679a8537d84c3852e026a4bdb39d0ee3b387411a91e8f6abafdc1ad<Q>,<Q>name<Q>:<Q>Ownbit<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://ownbit.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>83b291fa-1a08-4871-3ddb-8faa8be6f200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/83b291fa-1a08-4871-3ddb-8faa8be6f200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/83b291fa-1a08-4871-3ddb-8faa8be6f200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/83b291fa-1a08-4871-3ddb-8faa8be6f200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/es/app/ownbit-btc-eth-cold-wallet/id1321798216<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.bitbill.www<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Ownbit<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>244a0d93a45df0d0501a9cb9cdfb4e91aa750cfd4fc88f6e97a54d8455a76f5c<Q>:{<Q>id<Q>:<Q>244a0d93a45df0d0501a9cb9cdfb4e91aa750cfd4fc88f6e97a54d8455a76f5c<Q>,<Q>name<Q>:<Q>EasyPocket<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://easypocket.app/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a9fcf1d1-3b2c-4226-0fb9-a6c8f2647300<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a9fcf1d1-3b2c-4226-0fb9-a6c8f2647300/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a9fcf1d1-3b2c-4226-0fb9-a6c8f2647300/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a9fcf1d1-3b2c-4226-0fb9-a6c8f2647300/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/id1517338927<Q>,<Q>android<Q>:<Q>https://easypocket.app/<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>easypocket:<Q>,<Q>universal<Q>:<Q>https://wallet.easypocket.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>EasyPocket<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#115bfb<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>881946407ff22a32ec0e42b2cd31ea5dab52242dc3648d777b511a0440d59efb<Q>:{<Q>id<Q>:<Q>881946407ff22a32ec0e42b2cd31ea5dab52242dc3648d777b511a0440d59efb<Q>,<Q>name<Q>:<Q>Bridge Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://mtpelerin.com/bridge-wallet<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>989d504f-93db-4ca6-c00a-9d1faf177d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/989d504f-93db-4ca6-c00a-9d1faf177d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/989d504f-93db-4ca6-c00a-9d1faf177d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/989d504f-93db-4ca6-c00a-9d1faf177d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/bridge-wallet/id1481859680<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.mtpelerin.bridge<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>bridgewallet:<Q>,<Q>universal<Q>:<Q>https://bridge.mtpelerin.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Bridge<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>3b0e861b3a57e98325b82ab687fe0a712c81366d521ceec49eebc35591f1b5d1<Q>:{<Q>id<Q>:<Q>3b0e861b3a57e98325b82ab687fe0a712c81366d521ceec49eebc35591f1b5d1<Q>,<Q>name<Q>:<Q>SparkPoint<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://sparkpoint.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>34c4f901-70de-4507-e7a0-bc7887843000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/34c4f901-70de-4507-e7a0-bc7887843000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/34c4f901-70de-4507-e7a0-bc7887843000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/34c4f901-70de-4507-e7a0-bc7887843000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.sparkpoint<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>sparkpoint:<Q>,<Q>universal<Q>:<Q>https://sparkpoint.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>SparkPoint<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#14435f<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>ca86f48760bf5f84dcd6b1daca0fd55e2aa073ecf46453ba8a1db0b2e8e85ac1<Q>:{<Q>id<Q>:<Q>ca86f48760bf5f84dcd6b1daca0fd55e2aa073ecf46453ba8a1db0b2e8e85ac1<Q>,<Q>name<Q>:<Q>ViaWallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://viawallet.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>ffc3ba49-2e6b-4baa-304d-ebb253f74700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ffc3ba49-2e6b-4baa-304d-ebb253f74700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ffc3ba49-2e6b-4baa-304d-ebb253f74700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ffc3ba49-2e6b-4baa-304d-ebb253f74700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/us/app/viawallet/id1462031389<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.viabtc.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>ViaWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>42d72b6b34411dfacdf5364c027979908f971fc60251a817622b7bdb44a03106<Q>:{<Q>id<Q>:<Q>42d72b6b34411dfacdf5364c027979908f971fc60251a817622b7bdb44a03106<Q>,<Q>name<Q>:<Q>BitKeep<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://bitkeep.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>0b32f06b-bce3-46a7-d445-0e77dc2ad800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0b32f06b-bce3-46a7-d445-0e77dc2ad800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0b32f06b-bce3-46a7-d445-0e77dc2ad800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0b32f06b-bce3-46a7-d445-0e77dc2ad800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/app/bitkeep/id1395301115<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.bitkeep.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>bitkeep:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>BitKeep<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>b642ab6de0fe5c7d1e4a2b2821c9c807a81d0f6fd42ee3a75e513ea16e91151c<Q>:{<Q>id<Q>:<Q>b642ab6de0fe5c7d1e4a2b2821c9c807a81d0f6fd42ee3a75e513ea16e91151c<Q>,<Q>name<Q>:<Q>Vision<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://vision-crypto.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>c279537a-920d-422c-6a65-8b3bd524c300<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c279537a-920d-422c-6a65-8b3bd524c300/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c279537a-920d-422c-6a65-8b3bd524c300/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c279537a-920d-422c-6a65-8b3bd524c300/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/id1500186931<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.eletac.tronwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Vision<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>38ee551a01e3c5af9d8a9715768861e4d642e2381a62245083f96672b5646c6b<Q>:{<Q>id<Q>:<Q>38ee551a01e3c5af9d8a9715768861e4d642e2381a62245083f96672b5646c6b<Q>,<Q>name<Q>:<Q>PEAKDEFI Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://peakdefi.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>744a3fbe-4261-4148-133e-49c5b58cb400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/744a3fbe-4261-4148-133e-49c5b58cb400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/744a3fbe-4261-4148-133e-49c5b58cb400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/744a3fbe-4261-4148-133e-49c5b58cb400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/peakdefi-wallet/id1526193527<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.peakdefiwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>peakdefiwallet:<Q>,<Q>universal<Q>:<Q>https://peakdefi.com/download<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>PEAKDEFI<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>7e90b95230bc462869bbb59f952273d89841e1c76bcc5319898e08c9f34bd4cd<Q>:{<Q>id<Q>:<Q>7e90b95230bc462869bbb59f952273d89841e1c76bcc5319898e08c9f34bd4cd<Q>,<Q>name<Q>:<Q>Unstoppable Wallet<Q>,<Q>description<Q>:null,<Q>homepage<Q>:<Q>https://unstoppable.money/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>,<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>5c38b22c-adb9-4899-3252-6e3d71458500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5c38b22c-adb9-4899-3252-6e3d71458500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5c38b22c-adb9-4899-3252-6e3d71458500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5c38b22c-adb9-4899-3252-6e3d71458500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/app/apple-store/id1447619907<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.horizontalsystems.bankwallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>moneyunstoppable:<Q>,<Q>universal<Q>:<Q>https://unstoppable.money<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Unstoppable<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#FFBE43<Q>,<Q>secondary<Q>:null}}},<Q>025247d02e1972362982f04c96c78e7c02c4b68a9ac2107c26fe2ebb85c317c0<Q>:{<Q>id<Q>:<Q>025247d02e1972362982f04c96c78e7c02c4b68a9ac2107c26fe2ebb85c317c0<Q>,<Q>name<Q>:<Q>HaloDeFi Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://halodefi.org/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>af664892-49ef-4045-642e-347a4e98ee00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/af664892-49ef-4045-642e-347a4e98ee00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/af664892-49ef-4045-642e-347a4e98ee00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/af664892-49ef-4045-642e-347a4e98ee00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.halodefi.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>halodefiwallet:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>HaloDeFi<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>d12b6e114af8c47a6eec19a576f1022032a5ee4f8cafee612049f4796c803c7e<Q>:{<Q>id<Q>:<Q>d12b6e114af8c47a6eec19a576f1022032a5ee4f8cafee612049f4796c803c7e<Q>,<Q>name<Q>:<Q>Dok Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://dokwallet.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>6886f45d-1451-41ec-ebc7-b18bebfc3c00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6886f45d-1451-41ec-ebc7-b18bebfc3c00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6886f45d-1451-41ec-ebc7-b18bebfc3c00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6886f45d-1451-41ec-ebc7-b18bebfc3c00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/il/app/dokwallet-crypto-wallet/id1533065700<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.dok.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Dok<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>3d56ed42374504f1bb2ba368094269eaea461c075ab796d504f354baac213dc5<Q>:{<Q>id<Q>:<Q>3d56ed42374504f1bb2ba368094269eaea461c075ab796d504f354baac213dc5<Q>,<Q>name<Q>:<Q>AT.Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://authentrend.com/at-wallet/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a5b7369b-d92c-41a4-0263-ca28f4597600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a5b7369b-d92c-41a4-0263-ca28f4597600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a5b7369b-d92c-41a4-0263-ca28f4597600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a5b7369b-d92c-41a4-0263-ca28f4597600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/at-wallet/id1479171310<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.authentrend.atwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>AT.Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>1e04cf5cddcd84edb1370b12eae1fcecedf125b77209fff80e7ef2a6d3c74719<Q>:{<Q>id<Q>:<Q>1e04cf5cddcd84edb1370b12eae1fcecedf125b77209fff80e7ef2a6d3c74719<Q>,<Q>name<Q>:<Q>Midas Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://midasprotocol.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>4b7268e6-47fb-46bc-6f3c-424f44695f00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4b7268e6-47fb-46bc-6f3c-424f44695f00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4b7268e6-47fb-46bc-6f3c-424f44695f00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4b7268e6-47fb-46bc-6f3c-424f44695f00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/us/app/midas-protocol-crypto-wallet/id1436698193<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.midasprotocol.wallet.android<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Midas<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>15d1d97de89526a3c259a235304a7c510c40cda3331f0f8433da860ecc528bef<Q>:{<Q>id<Q>:<Q>15d1d97de89526a3c259a235304a7c510c40cda3331f0f8433da860ecc528bef<Q>,<Q>name<Q>:<Q>Ellipal<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://ellipal.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>0a805e10-bfc0-4d02-d9c1-8cec88f0dc00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0a805e10-bfc0-4d02-d9c1-8cec88f0dc00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0a805e10-bfc0-4d02-d9c1-8cec88f0dc00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0a805e10-bfc0-4d02-d9c1-8cec88f0dc00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/ellipal/id1426179665<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.ellipal.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>ellipal:<Q>,<Q>universal<Q>:<Q>https://www.ellipal.com/<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Ellipal<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#307cf9<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>19ad8334f0f034f4176a95722b5746b539b47b37ce17a5abde4755956d05d44c<Q>:{<Q>id<Q>:<Q>19ad8334f0f034f4176a95722b5746b539b47b37ce17a5abde4755956d05d44c<Q>,<Q>name<Q>:<Q>Aktionariat<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://aktionariat.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>6d18e8ea-b536-4038-c5bf-94a499d5a400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6d18e8ea-b536-4038-c5bf-94a499d5a400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6d18e8ea-b536-4038-c5bf-94a499d5a400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6d18e8ea-b536-4038-c5bf-94a499d5a400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/ch/app/aktionariat/id1518326813<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.aktionariat.app<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>aktionariat:<Q>,<Q>universal<Q>:<Q>https://app.aktionariat.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Aktionariat<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000000<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>95501c1a07c8eb575cb28c753ab9044259546ebcefcd3645461086e49b671f5c<Q>:{<Q>id<Q>:<Q>95501c1a07c8eb575cb28c753ab9044259546ebcefcd3645461086e49b671f5c<Q>,<Q>name<Q>:<Q>Talken Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://talken.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>1afb5a3a-2da3-40ce-baf9-b416e7510600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1afb5a3a-2da3-40ce-baf9-b416e7510600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1afb5a3a-2da3-40ce-baf9-b416e7510600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1afb5a3a-2da3-40ce-baf9-b416e7510600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/talken/id1459475831<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.talken.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>talken-wallet:<Q>,<Q>universal<Q>:<Q>https://talken.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Talken<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>78640a74036794a5b7f8ea501887c168232723696db4231f54abd3fe524037b4<Q>:{<Q>id<Q>:<Q>78640a74036794a5b7f8ea501887c168232723696db4231f54abd3fe524037b4<Q>,<Q>name<Q>:<Q>XinFin XDC Network<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://www.xinfin.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>bee71890-cdbe-4a9a-0d51-6cc75078f600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bee71890-cdbe-4a9a-0d51-6cc75078f600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bee71890-cdbe-4a9a-0d51-6cc75078f600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bee71890-cdbe-4a9a-0d51-6cc75078f600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://xinfin.network/<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.xdcwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>XinFin<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>d612ddb7326d7d64428d035971b82247322a4ffcf126027560502eff4c02bd1c<Q>:{<Q>id<Q>:<Q>d612ddb7326d7d64428d035971b82247322a4ffcf126027560502eff4c02bd1c<Q>,<Q>name<Q>:<Q>Flare Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://flarewallet.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>dfd3a3d8-d386-4c47-ff26-5c24845d2e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dfd3a3d8-d386-4c47-ff26-5c24845d2e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dfd3a3d8-d386-4c47-ff26-5c24845d2e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dfd3a3d8-d386-4c47-ff26-5c24845d2e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/flare-wallet/id1496651406<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.flare<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>flare:<Q>,<Q>universal<Q>:<Q>https://flarewallet.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Flare<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1f243b<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>2ed796df33fdbde6a3ea6a47d3636b8341fe285038d844c7a78267b465b27028<Q>:{<Q>id<Q>:<Q>2ed796df33fdbde6a3ea6a47d3636b8341fe285038d844c7a78267b465b27028<Q>,<Q>name<Q>:<Q>KyberSwap<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://kyberswap.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>3abd1720-260e-495a-2e31-3d0b349e0d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3abd1720-260e-495a-2e31-3d0b349e0d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3abd1720-260e-495a-2e31-3d0b349e0d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3abd1720-260e-495a-2e31-3d0b349e0d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://kyberswap.com/<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>kyberswap:<Q>,<Q>universal<Q>:<Q>https://kyberswapnew.app.link<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>KyberSwap<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>6193353e17504afc4bb982ee743ab970cd5cf842a35ecc9b7de61c150cf291e0<Q>:{<Q>id<Q>:<Q>6193353e17504afc4bb982ee743ab970cd5cf842a35ecc9b7de61c150cf291e0<Q>,<Q>name<Q>:<Q>AToken Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://atoken.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d93fb51d-f2b1-4b4e-f741-7485461bd500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d93fb51d-f2b1-4b4e-f741-7485461bd500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d93fb51d-f2b1-4b4e-f741-7485461bd500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d93fb51d-f2b1-4b4e-f741-7485461bd500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/atoken-app/id1395835245<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=wallet.gem.com.atoken<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>atoken:<Q>,<Q>universal<Q>:<Q>https://www.atoken.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>AToken<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>4e6af4201658b52daad51a279bb363a08b3927e74c0f27abeca3b0110bddf0a9<Q>:{<Q>id<Q>:<Q>4e6af4201658b52daad51a279bb363a08b3927e74c0f27abeca3b0110bddf0a9<Q>,<Q>name<Q>:<Q>Tongue Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://www.tongue.fi<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>20bc4fdb-b9e6-429a-8cba-c233b3273000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/20bc4fdb-b9e6-429a-8cba-c233b3273000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/20bc4fdb-b9e6-429a-8cba-c233b3273000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/20bc4fdb-b9e6-429a-8cba-c233b3273000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/fr/app/tongue-wallet-for-defi-degens/id1534705854<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>tongue:<Q>,<Q>universal<Q>:<Q>https://www.tongue.fi<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Tongue<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ff3178<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>b13fcc7e3500a4580c9a5341ed64c49c17d7f864497881048eb160c089be5346<Q>:{<Q>id<Q>:<Q>b13fcc7e3500a4580c9a5341ed64c49c17d7f864497881048eb160c089be5346<Q>,<Q>name<Q>:<Q>RWallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://rsk.co/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a883229c-26cb-4c19-9b34-1f0ed4012a00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a883229c-26cb-4c19-9b34-1f0ed4012a00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a883229c-26cb-4c19-9b34-1f0ed4012a00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a883229c-26cb-4c19-9b34-1f0ed4012a00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/rwallet/id1489241342<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.rsk.rwallet.reactnative<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>rwallet:<Q>,<Q>universal<Q>:<Q>https://www.rwallet.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>RWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>13c6a06b733edf51784f669f508826b2ab0dc80122a8b5d25d84b17d94bbdf70<Q>:{<Q>id<Q>:<Q>13c6a06b733edf51784f669f508826b2ab0dc80122a8b5d25d84b17d94bbdf70<Q>,<Q>name<Q>:<Q>PlasmaPay<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://plasmapay.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>466c8fd0-fcec-4621-b94f-e91ce1439f00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/466c8fd0-fcec-4621-b94f-e91ce1439f00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/466c8fd0-fcec-4621-b94f-e91ce1439f00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/466c8fd0-fcec-4621-b94f-e91ce1439f00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/id1461735396<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.plasmapay.androidapp<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>plasmapay:<Q>,<Q>universal<Q>:<Q>https://plasmapay.com/<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>PlasmaPay<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>0aafbedfb8eb56dae59ecc37c9a5388509cf9c082635e3f752581cc7128a17c0<Q>:{<Q>id<Q>:<Q>0aafbedfb8eb56dae59ecc37c9a5388509cf9c082635e3f752581cc7128a17c0<Q>,<Q>name<Q>:<Q>O3Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://o3.network<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e1c7c6af-c731-463e-55f0-5e686e9f6200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e1c7c6af-c731-463e-55f0-5e686e9f6200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e1c7c6af-c731-463e-55f0-5e686e9f6200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e1c7c6af-c731-463e-55f0-5e686e9f6200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/app/id1528451572<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.o3.o3wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>o3wallet:<Q>,<Q>universal<Q>:<Q>https://o3.network<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>O3Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>761d3d98fd77bdb06e6c90092ee7071c6001e93401d05dcf2b007c1a6c9c222c<Q>:{<Q>id<Q>:<Q>761d3d98fd77bdb06e6c90092ee7071c6001e93401d05dcf2b007c1a6c9c222c<Q>,<Q>name<Q>:<Q>HashKey Me<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://me.hashkey.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>72734fac-9500-4c2c-81ba-678f7fc32700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/72734fac-9500-4c2c-81ba-678f7fc32700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/72734fac-9500-4c2c-81ba-678f7fc32700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/72734fac-9500-4c2c-81ba-678f7fc32700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/hashkey-me/id1547228803<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>hashme:<Q>,<Q>universal<Q>:<Q>https://me.hashkey.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>HashKey Me<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#fe8f3f<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>0a00cbe128dddd6e096ebb78533a2c16ed409152a377c1f61a6a5ea643a2d950<Q>:{<Q>id<Q>:<Q>0a00cbe128dddd6e096ebb78533a2c16ed409152a377c1f61a6a5ea643a2d950<Q>,<Q>name<Q>:<Q>Jade Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://jadewallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>8a6f6b6f-9e25-43d2-6cb8-42013579bd00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8a6f6b6f-9e25-43d2-6cb8-42013579bd00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8a6f6b6f-9e25-43d2-6cb8-42013579bd00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8a6f6b6f-9e25-43d2-6cb8-42013579bd00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/jade-wallet-bitcoin-defi/id1544207492<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Jade<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#212d66<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>c04ae532094873c054a6c9339746c39c9ba5839c4d5bb2a1d9db51f9e5e77266<Q>:{<Q>id<Q>:<Q>c04ae532094873c054a6c9339746c39c9ba5839c4d5bb2a1d9db51f9e5e77266<Q>,<Q>name<Q>:<Q>Guarda Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://guarda.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>0142b5f2-2006-465f-fe0e-2021225d8c00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0142b5f2-2006-465f-fe0e-2021225d8c00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0142b5f2-2006-465f-fe0e-2021225d8c00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0142b5f2-2006-465f-fe0e-2021225d8c00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/apple-store/id1442083982?mt=8<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.crypto.multiwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Guarda<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#fff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>ffa139f74d1c8ebbb748cf0166f92d886e8c81b521c2193aa940e00626f4e215<Q>:{<Q>id<Q>:<Q>ffa139f74d1c8ebbb748cf0166f92d886e8c81b521c2193aa940e00626f4e215<Q>,<Q>name<Q>:<Q>Defiant<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://defiantapp.tech/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:42<Q>,<Q>eip155:30<Q>,<Q>eip155:31<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>026462e7-09a3-47f6-6b46-49df18133b00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/026462e7-09a3-47f6-6b46-49df18133b00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/026462e7-09a3-47f6-6b46-49df18133b00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/026462e7-09a3-47f6-6b46-49df18133b00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=ar.com.andinasmart.defiant<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>defiantapp:<Q>,<Q>universal<Q>:<Q>https://defiantapp.tech<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Defiant<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#274A60<Q>,<Q>secondary<Q>:<Q>#65ef9d<Q>}}},<Q>1ce6dae0fea7114846382391d946784d95d9032460a857bb23b55bd9807259d1<Q>:{<Q>id<Q>:<Q>1ce6dae0fea7114846382391d946784d95d9032460a857bb23b55bd9807259d1<Q>,<Q>name<Q>:<Q>Trustee Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://trusteeglobal.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>2432f3c2-83f1-486b-6081-d03acc33e000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2432f3c2-83f1-486b-6081-d03acc33e000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2432f3c2-83f1-486b-6081-d03acc33e000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2432f3c2-83f1-486b-6081-d03acc33e000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/trustee-wallet-bitcoin-wallet/id1462924276<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.trusteewallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Trustee<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>be6607b0a4093c0443bfe9c19ab30c99c91d2638866c99a6a16a71d3c1df78f8<Q>:{<Q>id<Q>:<Q>be6607b0a4093c0443bfe9c19ab30c99c91d2638866c99a6a16a71d3c1df78f8<Q>,<Q>name<Q>:<Q>CoinUs<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://coinus.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>1f92f9f9-08b9-4eca-4d75-425ce3d50100<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1f92f9f9-08b9-4eca-4d75-425ce3d50100/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1f92f9f9-08b9-4eca-4d75-425ce3d50100/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1f92f9f9-08b9-4eca-4d75-425ce3d50100/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/apple-store/id1367339746<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.theblockchain.coinus.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>CoinUs<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>ca151c4caeec5f9cc02ef03e498cb38c02ee5e498a8db13e853315077a5b45dc<Q>:{<Q>id<Q>:<Q>ca151c4caeec5f9cc02ef03e498cb38c02ee5e498a8db13e853315077a5b45dc<Q>,<Q>name<Q>:<Q>cmorq<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://cmorq.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>4e9f4558-32a2-46c9-be37-4926a6e95100<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4e9f4558-32a2-46c9-be37-4926a6e95100/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4e9f4558-32a2-46c9-be37-4926a6e95100/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4e9f4558-32a2-46c9-be37-4926a6e95100/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/cmorq/id1530022030<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>cmorq<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>d01c7758d741b363e637a817a09bcf579feae4db9f5bb16f599fdd1f66e2f974<Q>:{<Q>id<Q>:<Q>d01c7758d741b363e637a817a09bcf579feae4db9f5bb16f599fdd1f66e2f974<Q>,<Q>name<Q>:<Q>Valora<Q>,<Q>description<Q>:<Q>Valora is a mobile crypto wallet that enables global payments and easy access to decentralized apps.<Q>,<Q>homepage<Q>:<Q>https://valoraapp.com<Q>,<Q>chains<Q>:[<Q>eip155:42220<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a03bfa44-ce98-4883-9b2a-75e2b68f5700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a03bfa44-ce98-4883-9b2a-75e2b68f5700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a03bfa44-ce98-4883-9b2a-75e2b68f5700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a03bfa44-ce98-4883-9b2a-75e2b68f5700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/app/id1520414263<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=co.clabs.valora<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>celo:<Q>,<Q>universal<Q>:<Q>https://valoraapp.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Valora<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#42D689<Q>,<Q>secondary<Q>:<Q>#FBCC5C<Q>}}},<Q>e05615ed22df39c8d9b99ea38b45d4accb103fcef9cfa5d5edd38f5839b5182e<Q>:{<Q>id<Q>:<Q>e05615ed22df39c8d9b99ea38b45d4accb103fcef9cfa5d5edd38f5839b5182e<Q>,<Q>name<Q>:<Q>QuiverX<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://quiverx.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>67acf8ad-da61-4b7f-609b-57224fb8b100<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/67acf8ad-da61-4b7f-609b-57224fb8b100/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/67acf8ad-da61-4b7f-609b-57224fb8b100/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/67acf8ad-da61-4b7f-609b-57224fb8b100/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/quiverx/id1558993212<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.quiverx.app<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>QuiverX<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>36d854b702817e228d5c853c528d7bdb46f4bb041d255f67b82eb47111e5676b<Q>:{<Q>id<Q>:<Q>36d854b702817e228d5c853c528d7bdb46f4bb041d255f67b82eb47111e5676b<Q>,<Q>name<Q>:<Q>Celo Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://celowallet.app/<Q>,<Q>chains<Q>:[<Q>eip155:42220<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>482c9981-61c0-4782-84ec-c80fd997da00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/482c9981-61c0-4782-84ec-c80fd997da00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/482c9981-61c0-4782-84ec-c80fd997da00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/482c9981-61c0-4782-84ec-c80fd997da00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://celowallet.app/<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>celowallet:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Celo Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#35D07F<Q>,<Q>secondary<Q>:<Q>#FBCC5C<Q>}}},<Q>717911f4db0c5eda0e02e76ed179b7940ba1eefffdfb3c9e6540696226860da0<Q>:{<Q>id<Q>:<Q>717911f4db0c5eda0e02e76ed179b7940ba1eefffdfb3c9e6540696226860da0<Q>,<Q>name<Q>:<Q>Elastos Essentials<Q>,<Q>description<Q>:<Q>Your Web3 Crypto and W3C Identity wallet<Q>,<Q>homepage<Q>:<Q>https://elastos.org<Q>,<Q>chains<Q>:[<Q>eip155:20<Q>,<Q>eip155:21<Q>,<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:22<Q>,<Q>eip155:23<Q>,<Q>eip155:40<Q>,<Q>eip155:41<Q>,<Q>eip155:56<Q>,<Q>eip155:97<Q>,<Q>eip155:128<Q>,<Q>eip155:137<Q>,<Q>eip155:42161<Q>,<Q>eip155:43114<Q>,<Q>eip155:43113<Q>,<Q>eip155:250<Q>,<Q>eip155:4002<Q>,<Q>eip155:32659<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>04a6bfed-d80e-4f7b-0516-261f86aa4000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/04a6bfed-d80e-4f7b-0516-261f86aa4000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/04a6bfed-d80e-4f7b-0516-261f86aa4000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/04a6bfed-d80e-4f7b-0516-261f86aa4000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/elastos-essentials/id1568931743<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=org.elastos.essentials.app<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>essentials:<Q>,<Q>universal<Q>:<Q>https://essentials.elastos.net<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Essentials<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000000<Q>,<Q>secondary<Q>:<Q>#ffffff<Q>}}},<Q>c20b97dd1679625f4eb0bccd727c80746cb13bd97208b0c8e62c89cfd1d4b9cc<Q>:{<Q>id<Q>:<Q>c20b97dd1679625f4eb0bccd727c80746cb13bd97208b0c8e62c89cfd1d4b9cc<Q>,<Q>name<Q>:<Q>fuse.cash<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://fuse.cash/<Q>,<Q>chains<Q>:[<Q>eip155:122<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>71828267-72d6-4680-e144-265e6dc1e400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/71828267-72d6-4680-e144-265e6dc1e400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/71828267-72d6-4680-e144-265e6dc1e400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/71828267-72d6-4680-e144-265e6dc1e400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/fuse-cash/id1559937899<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.fuse.cash<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>fuse.cash:<Q>,<Q>universal<Q>:<Q>https://app.fuse.cash/<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>fuse.cash<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#37D888<Q>,<Q>secondary<Q>:<Q>#37D888<Q>}}},<Q>fbc8d86ad914ebd733fec4812b4b7af5ca709fdd9e75a930115e5baa02c4ef4c<Q>:{<Q>id<Q>:<Q>fbc8d86ad914ebd733fec4812b4b7af5ca709fdd9e75a930115e5baa02c4ef4c<Q>,<Q>name<Q>:<Q>Rabby<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://rabby.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7e82c795-574b-4e0f-1be7-d0677babed00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7e82c795-574b-4e0f-1be7-d0677babed00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7e82c795-574b-4e0f-1be7-d0677babed00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7e82c795-574b-4e0f-1be7-d0677babed00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://chrome.google.com/webstore/detail/empty-title/acmacodkjbdgmoleebolmdjonilkdbch<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Rabby<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>9d6c614d1995741d5313f1f3dbf1f66dcba694de782087d13b8721822502692f<Q>:{<Q>id<Q>:<Q>9d6c614d1995741d5313f1f3dbf1f66dcba694de782087d13b8721822502692f<Q>,<Q>name<Q>:<Q>Stasis<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://stasis.net/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7ae753bc-a754-450c-2d90-2c5521734400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7ae753bc-a754-450c-2d90-2c5521734400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7ae753bc-a754-450c-2d90-2c5521734400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7ae753bc-a754-450c-2d90-2c5521734400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/stasis-wallet/id1371949230<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.stasis.stasiswallet&hl=ru&gl=US<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>stasis:<Q>,<Q>universal<Q>:<Q>https://app.stasis.net<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Stasis<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#2468E4<Q>,<Q>secondary<Q>:<Q>#FFFFFF<Q>}}},<Q>a6ffb821a3c32d36fc9d29e29c2ff79a0cd1db0bca453714777846ddf3fdff76<Q>:{<Q>id<Q>:<Q>a6ffb821a3c32d36fc9d29e29c2ff79a0cd1db0bca453714777846ddf3fdff76<Q>,<Q>name<Q>:<Q>JulWallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://justliquidity.org/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:5<Q>,<Q>eip155:42<Q>,<Q>eip155:56<Q>,<Q>eip155:88<Q>,<Q>eip155:128<Q>,<Q>eip155:137<Q>,<Q>eip155:250<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>cabd50de-22fa-487b-ce68-2c63de8bb800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cabd50de-22fa-487b-ce68-2c63de8bb800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cabd50de-22fa-487b-ce68-2c63de8bb800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cabd50de-22fa-487b-ce68-2c63de8bb800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://chrome.google.com/webstore/detail/julwallet/fgkldlaelopgncepgpncndebgdddikii?hl=en-US<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/julwallet/id1555914591<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=julwallet.com<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>julwallet:<Q>,<Q>universal<Q>:<Q>https://justliquidity.org<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>JulWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#c84ce4<Q>,<Q>secondary<Q>:<Q>#3f00ff<Q>}}},<Q>2f010ad2b0777998e950b5d72969e3b6a7090f0dde59b509ce9a41a5d1b2f9f4<Q>:{<Q>id<Q>:<Q>2f010ad2b0777998e950b5d72969e3b6a7090f0dde59b509ce9a41a5d1b2f9f4<Q>,<Q>name<Q>:<Q>f(x) Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://functionx.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>4d5c5c70-5abb-43ba-fc5e-577b6e403300<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4d5c5c70-5abb-43ba-fc5e-577b6e403300/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4d5c5c70-5abb-43ba-fc5e-577b6e403300/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4d5c5c70-5abb-43ba-fc5e-577b6e403300/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/f-x-wallet-by-pundi-x-labs/id1504798360<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.pundix.functionx<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>f(x) Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>96bf9b7141215037f4a9f567ce536b1eb2836fb71cc67c07ba73f3c3eadc53d6<Q>:{<Q>id<Q>:<Q>96bf9b7141215037f4a9f567ce536b1eb2836fb71cc67c07ba73f3c3eadc53d6<Q>,<Q>name<Q>:<Q>Bull App<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://getbull.app/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>b3c42bfd-5078-4616-a2ad-e4e322bbf600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b3c42bfd-5078-4616-a2ad-e4e322bbf600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b3c42bfd-5078-4616-a2ad-e4e322bbf600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b3c42bfd-5078-4616-a2ad-e4e322bbf600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/id1558608737<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=crypto.wallet.chainapp<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Bull App<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>fc65284c7e407387d732fbfe5d57ed378a89db28c3559ada079f9097d43d2575<Q>:{<Q>id<Q>:<Q>fc65284c7e407387d732fbfe5d57ed378a89db28c3559ada079f9097d43d2575<Q>,<Q>name<Q>:<Q>Anybit<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://anybit.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>6aac5fb1-d400-4e81-4709-bef8b2c00900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6aac5fb1-d400-4e81-4709-bef8b2c00900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6aac5fb1-d400-4e81-4709-bef8b2c00900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6aac5fb1-d400-4e81-4709-bef8b2c00900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://www.anybit.io/#/download_ios<Q>,<Q>android<Q>:<Q>https://anybit-1254113200.cos.ap-shanghai.myqcloud.com/release/anybit-3.0.2.apk<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Anybit<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>ee97613ab75c89c3ddc6337952ff8b581ebf2466fc68e9bb601dca655dcef849<Q>:{<Q>id<Q>:<Q>ee97613ab75c89c3ddc6337952ff8b581ebf2466fc68e9bb601dca655dcef849<Q>,<Q>name<Q>:<Q>Bitpie<Q>,<Q>description<Q>:<Q>Bitpie Wallet<Q>,<Q>homepage<Q>:<Q>https://bitpiehk.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>f9b7c668-ed26-47f7-d8c9-7eadc7114800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f9b7c668-ed26-47f7-d8c9-7eadc7114800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f9b7c668-ed26-47f7-d8c9-7eadc7114800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f9b7c668-ed26-47f7-d8c9-7eadc7114800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://bitpiehk.com/ios<Q>,<Q>android<Q>:<Q>https://bitpiehk.com/android<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Bitpie<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>356fc3d6e801a76197a8b6f9bcac8ebe4f9e8ab8337012d4e453233983ff197a<Q>:{<Q>id<Q>:<Q>356fc3d6e801a76197a8b6f9bcac8ebe4f9e8ab8337012d4e453233983ff197a<Q>,<Q>name<Q>:<Q>Bitpie<Q>,<Q>description<Q>:<Q>Bitpie Wallet<Q>,<Q>homepage<Q>:<Q>https://bitpie.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>f9b7c668-ed26-47f7-d8c9-7eadc7114800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f9b7c668-ed26-47f7-d8c9-7eadc7114800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f9b7c668-ed26-47f7-d8c9-7eadc7114800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f9b7c668-ed26-47f7-d8c9-7eadc7114800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/bitpie-wallet/id1481314229<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.bitpie<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>bitpie:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Bitpie<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:<Q>#1a24ff<Q>}}},<Q>dea4ab675289353a508956c4f821bc4e9141d9f99a5d244ae7af92b4d575db6d<Q>:{<Q>id<Q>:<Q>dea4ab675289353a508956c4f821bc4e9141d9f99a5d244ae7af92b4d575db6d<Q>,<Q>name<Q>:<Q>Minerva Wallet<Q>,<Q>description<Q>:<Q>Minerva Wallet is a user friendly app for sovereign identities, data and money.<Q>,<Q>homepage<Q>:<Q>https://minerva.digital/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:5<Q>,<Q>eip155:22<Q>,<Q>eip155:42<Q>,<Q>eip155:56<Q>,<Q>eip155:77<Q>,<Q>eip155:97<Q>,<Q>eip155:99<Q>,<Q>eip155:100<Q>,<Q>eip155:137<Q>,<Q>eip155:80001<Q>,<Q>eip155:246529<Q>,<Q>eip155:246785<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7d5f3710-7c2c-49fc-7893-bacd3f384000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7d5f3710-7c2c-49fc-7893-bacd3f384000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7d5f3710-7c2c-49fc-7893-bacd3f384000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7d5f3710-7c2c-49fc-7893-bacd3f384000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=digital.minerva<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Minerva<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#443ad2<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>3cdf74643ac60e3f1d5807cb3dfa9d843793a4d1f3a233066fd35b61e72d42b9<Q>:{<Q>id<Q>:<Q>3cdf74643ac60e3f1d5807cb3dfa9d843793a4d1f3a233066fd35b61e72d42b9<Q>,<Q>name<Q>:<Q>ArchiPage<Q>,<Q>description<Q>:<Q>ArchiPage Wallet<Q>,<Q>homepage<Q>:<Q>https://criptoarchi.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>b62c4f22-e781-4ca9-5c01-ef7cd9d23400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b62c4f22-e781-4ca9-5c01-ef7cd9d23400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b62c4f22-e781-4ca9-5c01-ef7cd9d23400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b62c4f22-e781-4ca9-5c01-ef7cd9d23400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.archipage<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>archipage:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>ArchiPage<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>76745388a50e6fea982c4dee2a3ad61a8aa417668be870754689caa8a7506c93<Q>:{<Q>id<Q>:<Q>76745388a50e6fea982c4dee2a3ad61a8aa417668be870754689caa8a7506c93<Q>,<Q>name<Q>:<Q>Tangem<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://tangem.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>8a5b6e94-e378-458d-bf2e-017cc7958e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8a5b6e94-e378-458d-bf2e-017cc7958e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8a5b6e94-e378-458d-bf2e-017cc7958e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8a5b6e94-e378-458d-bf2e-017cc7958e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/uz/app/tangem/id1354868448<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.tangem.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>tangem:<Q>,<Q>universal<Q>:<Q>https://app.tangem.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Tangem<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#FFF<Q>,<Q>secondary<Q>:<Q>#000<Q>}}},<Q>67d728ec0a4d650a7a5b28390df634b86f775efc5075123aab537fc7fc53f045<Q>:{<Q>id<Q>:<Q>67d728ec0a4d650a7a5b28390df634b86f775efc5075123aab537fc7fc53f045<Q>,<Q>name<Q>:<Q>Chainge Finance<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://chainge.finance<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:66<Q>,<Q>eip155:128<Q>,<Q>eip155:137<Q>,<Q>eip155:250<Q>,<Q>eip155:32659<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>addaaf64-cf13-46ef-a022-d97189156f00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/addaaf64-cf13-46ef-a022-d97189156f00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/addaaf64-cf13-46ef-a022-d97189156f00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/addaaf64-cf13-46ef-a022-d97189156f00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/cn/app/id1578987516<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.fusion.chainge.android<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Chainge<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>9d93c497dc5c835abd1ddd3c5d388eaf54b995b49573340d9580b366259b7972<Q>:{<Q>id<Q>:<Q>9d93c497dc5c835abd1ddd3c5d388eaf54b995b49573340d9580b366259b7972<Q>,<Q>name<Q>:<Q>ioPay<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://iotex.io/<Q>,<Q>chains<Q>:[<Q>eip155:4689<Q>,<Q>eip155:4690<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>aa40d575-f7f4-4aa6-12c4-c8f055ad0800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/aa40d575-f7f4-4aa6-12c4-c8f055ad0800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/aa40d575-f7f4-4aa6-12c4-c8f055ad0800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/aa40d575-f7f4-4aa6-12c4-c8f055ad0800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://iopay-wallet.iotex.io/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/apple-store/id1478086371<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.iotex.iopay.gp<Q>,<Q>mac<Q>:<Q>https://github.com/iotexproject/iotex-explorer/releases<Q>,<Q>windows<Q>:<Q>https://github.com/iotexproject/iotex-explorer/releases<Q>,<Q>linux<Q>:<Q>https://github.com/iotexproject/iotex-explorer/releases<Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>ioPay<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>0105fbfd71efd3b19dde8060d6c9d959f555c078515a47b1a29cc9b67e6a4531<Q>:{<Q>id<Q>:<Q>0105fbfd71efd3b19dde8060d6c9d959f555c078515a47b1a29cc9b67e6a4531<Q>,<Q>name<Q>:<Q>Coinhub<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://coinhub.org/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>cd7d6974-739b-46d7-bd10-604222e16e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cd7d6974-739b-46d7-bd10-604222e16e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cd7d6974-739b-46d7-bd10-604222e16e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cd7d6974-739b-46d7-bd10-604222e16e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/tn/app/coinhub/id1567786851<Q>,<Q>android<Q>:<Q>https://static.coinhub.org/package/Coinhub_v1.2.0.apk<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Coinhub<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>b397a6d7fc2eb4895579d0fa043d66bbb0396e40237876d6d9f294f41cf755ac<Q>:{<Q>id<Q>:<Q>b397a6d7fc2eb4895579d0fa043d66bbb0396e40237876d6d9f294f41cf755ac<Q>,<Q>name<Q>:<Q>Go Pocket<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://gopocket.finance/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>2494a686-3e07-4e9b-15ef-3605dca32a00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2494a686-3e07-4e9b-15ef-3605dca32a00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2494a686-3e07-4e9b-15ef-3605dca32a00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2494a686-3e07-4e9b-15ef-3605dca32a00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/go-pocket/id1576488287?l=en<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.gopocket<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Go Pocket<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>addb6cfece8fe6d2e7039baf5b2ba3249da48957b08bcc877a2e32eaffa6e7aa<Q>:{<Q>id<Q>:<Q>addb6cfece8fe6d2e7039baf5b2ba3249da48957b08bcc877a2e32eaffa6e7aa<Q>,<Q>name<Q>:<Q>Wallet 3<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://wallet3.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d740b48c-2b55-4a27-b5f5-d2188200ca00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d740b48c-2b55-4a27-b5f5-d2188200ca00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d740b48c-2b55-4a27-b5f5-d2188200ca00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d740b48c-2b55-4a27-b5f5-d2188200ca00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q>https://wallet3.io/<Q>,<Q>windows<Q>:<Q>https://wallet3.io/<Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q>wallet3:<Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Wallet 3<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#6186ff<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>1bc1d561a2a38a2ff0c4dfb21c2236f7335084f822cd5de016bd4725fed389df<Q>:{<Q>id<Q>:<Q>1bc1d561a2a38a2ff0c4dfb21c2236f7335084f822cd5de016bd4725fed389df<Q>,<Q>name<Q>:<Q>yiToken<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://yitoken.im<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>b5cc13d4-2a58-4142-08dd-5596ab253800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b5cc13d4-2a58-4142-08dd-5596ab253800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b5cc13d4-2a58-4142-08dd-5596ab253800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/b5cc13d4-2a58-4142-08dd-5596ab253800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://testflight.apple.com/join/vwyDi2pn<Q>,<Q>android<Q>:<Q>https://app.iwishbest.com/apps/yiToken.apk<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>yitoken:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>yiToken<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#2c4cfc<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>416a17a68726c10896a46e9ecbb25eaa0e342611b2387d4787902bf1984b68e9<Q>:{<Q>id<Q>:<Q>416a17a68726c10896a46e9ecbb25eaa0e342611b2387d4787902bf1984b68e9<Q>,<Q>name<Q>:<Q>DID Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://abtwallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>673b7f3b-a555-4327-f9b7-fefa535bc500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/673b7f3b-a555-4327-f9b7-fefa535bc500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/673b7f3b-a555-4327-f9b7-fefa535bc500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/673b7f3b-a555-4327-f9b7-fefa535bc500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/app/id1460083542<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.arcblock.wallet.app.product<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>abt:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>DID<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#4598FA<Q>,<Q>secondary<Q>:<Q>#53A8F4<Q>}}},<Q>c1abb33fa718be77d96c56a3f9720400e9c5de2f79ef50ad6a2c19b1c28659f3<Q>:{<Q>id<Q>:<Q>c1abb33fa718be77d96c56a3f9720400e9c5de2f79ef50ad6a2c19b1c28659f3<Q>,<Q>name<Q>:<Q>StarBase<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://deploystarship.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>2bd78dfd-37d9-4334-8afb-17544b85f200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2bd78dfd-37d9-4334-8afb-17544b85f200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2bd78dfd-37d9-4334-8afb-17544b85f200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2bd78dfd-37d9-4334-8afb-17544b85f200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.starbase.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>StarBase<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>29449bf9d53ae02dd64a6719915c65882d7d634cca2d2de19586c772e6cbe759<Q>:{<Q>id<Q>:<Q>29449bf9d53ae02dd64a6719915c65882d7d634cca2d2de19586c772e6cbe759<Q>,<Q>name<Q>:<Q>Shinobi Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://shinobi-wallet.net/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>f5cf868c-5347-4d5e-e80f-c6ece8fcb600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f5cf868c-5347-4d5e-e80f-c6ece8fcb600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f5cf868c-5347-4d5e-e80f-c6ece8fcb600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f5cf868c-5347-4d5e-e80f-c6ece8fcb600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/fgraph/id1572609905<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.fgraph.shinobiwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Shinobi Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>403308a7b7d5da26558cb3658d963d9a87648940a38cc04143f223815d631aa5<Q>:{<Q>id<Q>:<Q>403308a7b7d5da26558cb3658d963d9a87648940a38cc04143f223815d631aa5<Q>,<Q>name<Q>:<Q>GD Wallet<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://open.seaio.cc/<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>5363dc86-3de9-4197-c6ed-e8167b673a00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5363dc86-3de9-4197-c6ed-e8167b673a00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5363dc86-3de9-4197-c6ed-e8167b673a00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5363dc86-3de9-4197-c6ed-e8167b673a00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://dapp-h5.seaio.cc/newsDetail/#/downLoad?lang=ZH<Q>,<Q>android<Q>:<Q>https://dapp-h5.seaio.cc/newsDetail/#/downLoad?lang=ZH<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>GDWalletWC:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>GD Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>2235b648bdf382bc1a6960f1db8eda5c71f65a1996e6cb549d24783ca58a2903<Q>:{<Q>id<Q>:<Q>2235b648bdf382bc1a6960f1db8eda5c71f65a1996e6cb549d24783ca58a2903<Q>,<Q>name<Q>:<Q>Binana<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://binana.vip/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>65a60845-8fe3-4146-2688-586e4dc68a00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/65a60845-8fe3-4146-2688-586e4dc68a00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/65a60845-8fe3-4146-2688-586e4dc68a00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/65a60845-8fe3-4146-2688-586e4dc68a00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://binana.vip/<Q>,<Q>android<Q>:<Q>https://binana.vip/<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q>https://master.tokenone.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Binana<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>a0718f5fb1493e4aaac72cff62d162cb85db40ed68fd8700298f36f1d5c4b73d<Q>:{<Q>id<Q>:<Q>a0718f5fb1493e4aaac72cff62d162cb85db40ed68fd8700298f36f1d5c4b73d<Q>,<Q>name<Q>:<Q>AirGap<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://airgap.it<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>cc530d2d-4d01-4f9f-cf38-14deb71fd600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cc530d2d-4d01-4f9f-cf38-14deb71fd600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cc530d2d-4d01-4f9f-cf38-14deb71fd600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cc530d2d-4d01-4f9f-cf38-14deb71fd600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://wallet.airgap.it<Q>,<Q>ios<Q>:<Q>https://itunes.apple.com/us/app/airgap-wallet/id1420996542?l=de&ls=1&mt=8<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=it.airgap.wallet<Q>,<Q>mac<Q>:<Q>https://github.com/airgap-it/airgap-wallet/releases<Q>,<Q>windows<Q>:<Q>https://github.com/airgap-it/airgap-wallet/releases<Q>,<Q>linux<Q>:<Q>https://github.com/airgap-it/airgap-wallet/releases<Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>airgap-wallet:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>AirGap<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#50249f<Q>,<Q>secondary<Q>:<Q>#00e8d0<Q>}}},<Q>76428179ce9213ab6f8f49923310efcf5eea089764196c7a2018bea8afcd6603<Q>:{<Q>id<Q>:<Q>76428179ce9213ab6f8f49923310efcf5eea089764196c7a2018bea8afcd6603<Q>,<Q>name<Q>:<Q>PayTube<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://paytube.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>58230f60-6c7b-400c-ab96-cb1fd0391700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/58230f60-6c7b-400c-ab96-cb1fd0391700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/58230f60-6c7b-400c-ab96-cb1fd0391700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/58230f60-6c7b-400c-ab96-cb1fd0391700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/paytube/id1574456998<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.paytube.wallet.zkwallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>PayTube<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>6464873279d46030c0b6b005b33da6be5ed57a752be3ef1f857dc10eaf8028aa<Q>:{<Q>id<Q>:<Q>6464873279d46030c0b6b005b33da6be5ed57a752be3ef1f857dc10eaf8028aa<Q>,<Q>name<Q>:<Q>SecuX<Q>,<Q>description<Q>:<Q>SecuX Hardware Wallet<Q>,<Q>homepage<Q>:<Q>https://secuxtech.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>6013a9a1-4a67-45bb-fc24-27c11eb13900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6013a9a1-4a67-45bb-fc24-27c11eb13900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6013a9a1-4a67-45bb-fc24-27c11eb13900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/6013a9a1-4a67-45bb-fc24-27c11eb13900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://wallet.secuxtech.com/secuxess/#/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/tw/app/secux-mobile/id1477437607<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.secux.mobile&hl=zh_TW&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>secux://<Q>,<Q>universal<Q>:<Q>https://wsweb.secuxtech.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q>secux://<Q>,<Q>universal<Q>:<Q>https://wsweb.secuxtech.com<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>SecuX<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>42049d5de7c9e88664bb4e13db614decb08a4ba4f0e6eefd5617f25d3cb2983b<Q>:{<Q>id<Q>:<Q>42049d5de7c9e88664bb4e13db614decb08a4ba4f0e6eefd5617f25d3cb2983b<Q>,<Q>name<Q>:<Q>BlockBank<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://blockbank.ai<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>5b2cc39f-bc4f-4ac1-b6d7-08bcc9066a00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5b2cc39f-bc4f-4ac1-b6d7-08bcc9066a00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5b2cc39f-bc4f-4ac1-b6d7-08bcc9066a00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/5b2cc39f-bc4f-4ac1-b6d7-08bcc9066a00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/blockbank/id1460965460<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.aibb.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>BBank<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#f26422<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>d864d048f82084fac88d386c32b3261513ed7b5d1d4b52f96f91022059e984f9<Q>:{<Q>id<Q>:<Q>d864d048f82084fac88d386c32b3261513ed7b5d1d4b52f96f91022059e984f9<Q>,<Q>name<Q>:<Q>Orange<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://orangewallet.app/<Q>,<Q>chains<Q>:[<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7fcbf9f0-0c0b-439d-3fdb-31b32c28df00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7fcbf9f0-0c0b-439d-3fdb-31b32c28df00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7fcbf9f0-0c0b-439d-3fdb-31b32c28df00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7fcbf9f0-0c0b-439d-3fdb-31b32c28df00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=org.orange.wallet<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>orangewallet:<Q>,<Q>universal<Q>:<Q>https://link.orangewallet.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Orange<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#EB5B56<Q>,<Q>secondary<Q>:<Q>#FFF<Q>}}},<Q>28ec413631bf709cc6aeb3d4e3fdd7be541d38ba852ac491840e2501eaa7cb4b<Q>:{<Q>id<Q>:<Q>28ec413631bf709cc6aeb3d4e3fdd7be541d38ba852ac491840e2501eaa7cb4b<Q>,<Q>name<Q>:<Q>NEFTiPEDiA<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://neftipedia.com/<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>3f503c40-d5f0-4430-b996-3126a9968c00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3f503c40-d5f0-4430-b996-3126a9968c00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3f503c40-d5f0-4430-b996-3126a9968c00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3f503c40-d5f0-4430-b996-3126a9968c00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.NEFTiPEDiA.mp<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>NEFTiPEDiA<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>2969649937a2a6c587e1391446d60e2e06b9c5196162a6aa70a0002292aa8d22<Q>:{<Q>id<Q>:<Q>2969649937a2a6c587e1391446d60e2e06b9c5196162a6aa70a0002292aa8d22<Q>,<Q>name<Q>:<Q>Avacus<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://avacus.cc/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e90bf788-03e3-48eb-e892-adfae0a61500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e90bf788-03e3-48eb-e892-adfae0a61500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e90bf788-03e3-48eb-e892-adfae0a61500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e90bf788-03e3-48eb-e892-adfae0a61500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/id1339638445<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.floortracks.avacus<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>avacus:<Q>,<Q>universal<Q>:<Q>https://avacus.app.link<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Avacus<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#f7a026<Q>,<Q>secondary<Q>:<Q>#fff<Q>}}},<Q>58cb8992bd89d9b90bf1e6208a9f6d3f4b2959fe81b4566bc5c4621f99db2499<Q>:{<Q>id<Q>:<Q>58cb8992bd89d9b90bf1e6208a9f6d3f4b2959fe81b4566bc5c4621f99db2499<Q>,<Q>name<Q>:<Q>Krystal<Q>,<Q>description<Q>:<Q>Krystal is a secure and simple way to access the best DeFi services on a single platform<Q>,<Q>homepage<Q>:<Q>https://krystal.app/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:43114<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>91449cb2-57b0-4bb6-481b-47d489f7a800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/91449cb2-57b0-4bb6-481b-47d489f7a800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/91449cb2-57b0-4bb6-481b-47d489f7a800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/91449cb2-57b0-4bb6-481b-47d489f7a800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/vn/app/krystal-one-platform-all-defi/id1558105691<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.kyrd.krystal<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>krystalWallet:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Krystal<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1de9b6<Q>,<Q>secondary<Q>:<Q>#000<Q>}}},<Q>dd8ee41915d967e547c80266e883d77ee808427405f4e8026a85ac1308104221<Q>:{<Q>id<Q>:<Q>dd8ee41915d967e547c80266e883d77ee808427405f4e8026a85ac1308104221<Q>,<Q>name<Q>:<Q>Linen<Q>,<Q>description<Q>:<Q><Q>,<Q>homepage<Q>:<Q>https://linen.app/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:42<Q>,<Q>eip155:137<Q>,<Q>eip155:80001<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>91458f54-aabe-44cf-4788-159ccc733600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/91458f54-aabe-44cf-4788-159ccc733600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/91458f54-aabe-44cf-4788-159ccc733600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/91458f54-aabe-44cf-4788-159ccc733600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/linen-app-investing-in-defi/id1480509067<Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q>https://linen.app/<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Linen<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#009fbe<Q>,<Q>secondary<Q>:<Q>#fff<Q>}}},<Q>2c81da3add65899baeac53758a07e652eea46dbb5195b8074772c62a77bbf568<Q>:{<Q>id<Q>:<Q>2c81da3add65899baeac53758a07e652eea46dbb5195b8074772c62a77bbf568<Q>,<Q>name<Q>:<Q>Ambire Wallet<Q>,<Q>description<Q>:<Q>Ambire Wallet is a full featured non-custodial open-source wallet focused on DeFi and ease of use.<Q>,<Q>homepage<Q>:<Q>https://ambire.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:43114<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>c39b3a16-1a38-4588-f089-cb7aeb584700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c39b3a16-1a38-4588-f089-cb7aeb584700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c39b3a16-1a38-4588-f089-cb7aeb584700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c39b3a16-1a38-4588-f089-cb7aeb584700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://wallet.walletconnect.com<Q>,<Q>ios<Q>:null,<Q>android<Q>:null,<Q>mac<Q>:null,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>ambire:<Q>,<Q>universal<Q>:<Q>https://mobile.ambire.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://wallet.ambire.com<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Ambire<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#aa6aff<Q>,<Q>secondary<Q>:<Q>#80ffdb<Q>}}},<Q>f2dcbeb246b4e4d37d748a7b2be54bbd93bdbe3e351d0badc1cbd3ea262d8466<Q>:{<Q>id<Q>:<Q>f2dcbeb246b4e4d37d748a7b2be54bbd93bdbe3e351d0badc1cbd3ea262d8466<Q>,<Q>name<Q>:<Q>BRISE Wallet<Q>,<Q>description<Q>:<Q>You can send, receive and store coins and many other cryptocurrencies and digital assets safely and securely.<Q>,<Q>homepage<Q>:<Q>https://bitgert.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>09a4e1d9-e4de-44fa-f248-5495ba9ab300<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/09a4e1d9-e4de-44fa-f248-5495ba9ab300/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/09a4e1d9-e4de-44fa-f248-5495ba9ab300/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/09a4e1d9-e4de-44fa-f248-5495ba9ab300/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:null,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.brise.wallet&hl=en_US&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>BRISE Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>2a527d1ee63ab8837381714b85f75d57217699c2b758a8d79bf6658f02db0578<Q>:{<Q>id<Q>:<Q>2a527d1ee63ab8837381714b85f75d57217699c2b758a8d79bf6658f02db0578<Q>,<Q>name<Q>:<Q>WATT ME<Q>,<Q>description<Q>:<Q>More than just a wallet Multifunctional app for storing, buying and professional crypto currency trading as well as NFT-tokens storing.<Q>,<Q>homepage<Q>:<Q>https://watt.me<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:137<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>38c619c4-5365-4de5-09b2-cdde8caf3600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/38c619c4-5365-4de5-09b2-cdde8caf3600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/38c619c4-5365-4de5-09b2-cdde8caf3600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/38c619c4-5365-4de5-09b2-cdde8caf3600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/ru/app/watt-me/id1594847330<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=watt.wallet&hl=ru&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>wattwallet://wallet-connect/<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>WATT ME<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#F5EB51<Q>,<Q>secondary<Q>:<Q>#282828<Q>}}},<Q>5859076ade608fbc4e9d3fe2f95e8527de80f8451ecbb1dced54ca84deae0dd6<Q>:{<Q>id<Q>:<Q>5859076ade608fbc4e9d3fe2f95e8527de80f8451ecbb1dced54ca84deae0dd6<Q>,<Q>name<Q>:<Q>Coingrig<Q>,<Q>description<Q>:<Q>A powerful crypto wallet for everyone. Private, secure and open source.<Q>,<Q>homepage<Q>:<Q>https://coingrig.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:43114<Q>,<Q>eip155:25<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>18e38e41-a387-4402-ca31-6d2d5eb91100<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/18e38e41-a387-4402-ca31-6d2d5eb91100/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/18e38e41-a387-4402-ca31-6d2d5eb91100/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/18e38e41-a387-4402-ca31-6d2d5eb91100/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/coingrig-crypto-btc-wallet/id1583464451<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.coingrig<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>coingrig://<Q>,<Q>universal<Q>:<Q>https://link.coingrig.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Coingrig<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>fd144d6a46656068f37fb1d59d3caf413651ea5f4a24025c9fd72a6bbe22d834<Q>:{<Q>id<Q>:<Q>fd144d6a46656068f37fb1d59d3caf413651ea5f4a24025c9fd72a6bbe22d834<Q>,<Q>name<Q>:<Q>Bycoin<Q>,<Q>description<Q>:<Q>Professional digital assets management platform<Q>,<Q>homepage<Q>:<Q>https://bycoin.im<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>980b0c5f-353d-4643-1ee8-d9264ec30000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/980b0c5f-353d-4643-1ee8-d9264ec30000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/980b0c5f-353d-4643-1ee8-d9264ec30000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/980b0c5f-353d-4643-1ee8-d9264ec30000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://itunes.apple.com/us/app/bycoin/id1442183969?l=zh&ls=1&mt=8<Q>,<Q>android<Q>:<Q>https://cdn.bystack.com/bycoin/apk/Bycoin.apk<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Bycoin<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>51334e444ea1ba3d23c96063b8600c94af89233bd3f8f3685123c46e0348766c<Q>:{<Q>id<Q>:<Q>51334e444ea1ba3d23c96063b8600c94af89233bd3f8f3685123c46e0348766c<Q>,<Q>name<Q>:<Q>Snowball<Q>,<Q>description<Q>:<Q>Gain access to high-yield stablecoin vaults and generate interest in real-time via DeFi<Q>,<Q>homepage<Q>:<Q>https://snowball.money/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>313faea4-af8c-41f4-0ed8-98be5d048e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/313faea4-af8c-41f4-0ed8-98be5d048e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/313faea4-af8c-41f4-0ed8-98be5d048e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/313faea4-af8c-41f4-0ed8-98be5d048e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/app/snowball-smart-defi-wallet/id1449662311<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=money.snowball.defi<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://app.snowball.exchange/app<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Snowball<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#292E39<Q>,<Q>secondary<Q>:null}}},<Q>792fbacfe787d67595dd4eb38ac308e14b3bbc810393db56f477a92e5ac8764b<Q>:{<Q>id<Q>:<Q>792fbacfe787d67595dd4eb38ac308e14b3bbc810393db56f477a92e5ac8764b<Q>,<Q>name<Q>:<Q>Assure<Q>,<Q>description<Q>:<Q>A decentralized wallet of digital currency<Q>,<Q>homepage<Q>:<Q>https://www.assurepro.io/<Q>,<Q>chains<Q>:[<Q>polkadot:91b171bb158e2d3848fa23a9f1c25182<Q>,<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:128<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>64db7104-c8b7-44ea-e102-11ce87124200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/64db7104-c8b7-44ea-e102-11ce87124200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/64db7104-c8b7-44ea-e102-11ce87124200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/64db7104-c8b7-44ea-e102-11ce87124200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>http://itunes.apple.com/app/id1604825026<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.neuxs.assure<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>assure://<Q>,<Q>universal<Q>:<Q>https://www.assurepro.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Assure<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>5864e2ced7c293ed18ac35e0db085c09ed567d67346ccb6f58a0327a75137489<Q>:{<Q>id<Q>:<Q>5864e2ced7c293ed18ac35e0db085c09ed567d67346ccb6f58a0327a75137489<Q>,<Q>name<Q>:<Q>Fireblocks<Q>,<Q>description<Q>:<Q>#1 Crypto and Digital Asset Platform for Institutions<Q>,<Q>homepage<Q>:<Q>https://www.fireblocks.com/<Q>,<Q>chains<Q>:[<Q>polkadot:91b171bb158e2d3848fa23a9f1c25182<Q>,<Q>polkadot:b0a8d493285c2df73290dfb7e61f870f<Q>,<Q>cosmos:columbus-4<Q>,<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:10<Q>,<Q>eip155:19<Q>,<Q>eip155:30<Q>,<Q>eip155:31<Q>,<Q>eip155:42<Q>,<Q>eip155:50<Q>,<Q>eip155:51<Q>,<Q>eip155:56<Q>,<Q>eip155:59<Q>,<Q>eip155:61<Q>,<Q>eip155:63<Q>,<Q>eip155:69<Q>,<Q>eip155:97<Q>,<Q>eip155:137<Q>,<Q>eip155:250<Q>,<Q>eip155:1285<Q>,<Q>eip155:1284<Q>,<Q>eip155:10000<Q>,<Q>eip155:10001<Q>,<Q>eip155:42161<Q>,<Q>eip155:42220<Q>,<Q>eip155:43113<Q>,<Q>eip155:43114<Q>,<Q>eip155:44787<Q>,<Q>eip155:62320<Q>,<Q>eip155:421611<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7e1514ba-932d-415d-1bdb-bccb6c2cbc00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7e1514ba-932d-415d-1bdb-bccb6c2cbc00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7e1514ba-932d-415d-1bdb-bccb6c2cbc00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7e1514ba-932d-415d-1bdb-bccb6c2cbc00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://console.fireblocks.io/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/fireblocks/id1439296596<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.fireblocks.client&gl=IL<Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Fireblocks<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>b265ce38b94d602957a0946673c59a99a15d69adda4317544fec7298ea2d1387<Q>:{<Q>id<Q>:<Q>b265ce38b94d602957a0946673c59a99a15d69adda4317544fec7298ea2d1387<Q>,<Q>name<Q>:<Q>SafeMoon<Q>,<Q>description<Q>:<Q>SafeMoon Wallet is the safe place to buy, sell, and swap your crypto.<Q>,<Q>homepage<Q>:<Q>https://safemoon.net/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>ecc31a8e-0ee9-49db-cc59-0876b7c35600<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ecc31a8e-0ee9-49db-cc59-0876b7c35600/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ecc31a8e-0ee9-49db-cc59-0876b7c35600/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ecc31a8e-0ee9-49db-cc59-0876b7c35600/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/safemoon/id1579735495<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=net.safemoon.androidwallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>safemoon:<Q>,<Q>universal<Q>:<Q>https://safemoon.net/<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>SafeMoon<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>bae2ab14ef450f307f24a395a3c2766a2ef8a9c0e61856985d23f6445e8db03f<Q>:{<Q>id<Q>:<Q>bae2ab14ef450f307f24a395a3c2766a2ef8a9c0e61856985d23f6445e8db03f<Q>,<Q>name<Q>:<Q>CryptnoxApp<Q>,<Q>description<Q>:<Q>IOS app to use our smartcards as hardware wallet via NFC<Q>,<Q>homepage<Q>:<Q>https://cryptnox.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>2947b7c8-8966-4485-a98d-25fe43c16700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2947b7c8-8966-4485-a98d-25fe43c16700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2947b7c8-8966-4485-a98d-25fe43c16700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2947b7c8-8966-4485-a98d-25fe43c16700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/app/id1583011693<Q>,<Q>android<Q>:null,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>CryptnoxApp<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>3b9f67c2c0887f71e4f9ba1bd2bf5b4eb6cda94419abd3f0c5c12931a60928b0<Q>:{<Q>id<Q>:<Q>3b9f67c2c0887f71e4f9ba1bd2bf5b4eb6cda94419abd3f0c5c12931a60928b0<Q>,<Q>name<Q>:<Q>Bitski<Q>,<Q>description<Q>:<Q>Most secured mobile wallet<Q>,<Q>homepage<Q>:<Q>https://bitski.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:4<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>94d94cb5-a94f-47cf-70e6-fe8d3f1c3700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/94d94cb5-a94f-47cf-70e6-fe8d3f1c3700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/94d94cb5-a94f-47cf-70e6-fe8d3f1c3700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/94d94cb5-a94f-47cf-70e6-fe8d3f1c3700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://wallet.bitski.com<Q>,<Q>ios<Q>:<Q>https://testflight.apple.com/join/L76HIQbD<Q>,<Q>android<Q>:null,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q>https://wallet.bitski.com/walletconnect<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Bitski<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>584538f059b079deecc80dface062cf40c33afd45dca02c7edca134a8225556d<Q>:{<Q>id<Q>:<Q>584538f059b079deecc80dface062cf40c33afd45dca02c7edca134a8225556d<Q>,<Q>name<Q>:<Q>Ballet Crypto<Q>,<Q>description<Q>:<Q>Ballet Crypto is the world’s first multi-currency, non-electronic cryptocurrency wallet.<Q>,<Q>homepage<Q>:<Q>https://www.balletcrypto.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>542094e6-70d6-4b0d-4c8f-b61cc2c38500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/542094e6-70d6-4b0d-4c8f-b61cc2c38500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/542094e6-70d6-4b0d-4c8f-b61cc2c38500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/542094e6-70d6-4b0d-4c8f-b61cc2c38500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/ballet-crypto/id1474912942<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.balletcrypto&hl=en_US&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:null,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>f2dca938b70ea7965ffbc3ef49f3e21701d1fc4f1c543d4b05801c126416466b<Q>:{<Q>id<Q>:<Q>f2dca938b70ea7965ffbc3ef49f3e21701d1fc4f1c543d4b05801c126416466b<Q>,<Q>name<Q>:<Q>FILWallet<Q>,<Q>description<Q>:<Q>专注于Web3.0生态的去中心化数字资产钱包<Q>,<Q>homepage<Q>:<Q>https://filwallet.co/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>f400f6c2-ca6c-487b-654d-e119af247500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f400f6c2-ca6c-487b-654d-e119af247500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f400f6c2-ca6c-487b-654d-e119af247500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f400f6c2-ca6c-487b-654d-e119af247500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/filwallet-io/id1572930901<Q>,<Q>android<Q>:<Q>https://filwallet.co/<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://h5.filwallet.co<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>FILWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>74f8092562bd79675e276d8b2062a83601a4106d30202f2d509195e30e19673d<Q>:{<Q>id<Q>:<Q>74f8092562bd79675e276d8b2062a83601a4106d30202f2d509195e30e19673d<Q>,<Q>name<Q>:<Q>Spot<Q>,<Q>description<Q>:<Q>Spot is a mobile & secure non-custodial wallet for Ethereum, Solana, Bitcoin, Tezos & NFTs. Access web3 & DeFi with WalletConnect.<Q>,<Q>homepage<Q>:<Q>https://www.spot-wallet.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>7d90b813-c44e-41c2-a105-d20b26c71000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7d90b813-c44e-41c2-a105-d20b26c71000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7d90b813-c44e-41c2-a105-d20b26c71000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/7d90b813-c44e-41c2-a105-d20b26c71000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/buy-bitcoin-spot-wallet-app/id1390560448<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.spot.spot<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>spot:<Q>,<Q>universal<Q>:<Q>https://spot.so/<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Spot<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#29C0FF<Q>,<Q>secondary<Q>:<Q>#29C0FF<Q>}}},<Q>e3787ea98d014ca77e2c3794db97c02ef8bcb39347705f5e79502a55434a1ecf<Q>:{<Q>id<Q>:<Q>e3787ea98d014ca77e2c3794db97c02ef8bcb39347705f5e79502a55434a1ecf<Q>,<Q>name<Q>:<Q>TK Finance<Q>,<Q>description<Q>:<Q>Entering the great Trustkeys Finance Ecosystem. Fast, safe, secure trading experience with Hybrid Change.<Q>,<Q>homepage<Q>:<Q>https://trustkeys.network/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:3<Q>,<Q>eip155:97<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>c4066f68-2247-49bf-ac8a-a677bfa81800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c4066f68-2247-49bf-ac8a-a677bfa81800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c4066f68-2247-49bf-ac8a-a677bfa81800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c4066f68-2247-49bf-ac8a-a677bfa81800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/vn/app/tk-finance/id1601968967<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.trustkeysnetwork<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>tk://<Q>,<Q>universal<Q>:<Q>https://trustkeys.network<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>TK Finance<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>f94a60403cdffa9a521dd12f9ec1004a887755c62ecf7bb4e9b8ee6806c26b58<Q>:{<Q>id<Q>:<Q>f94a60403cdffa9a521dd12f9ec1004a887755c62ecf7bb4e9b8ee6806c26b58<Q>,<Q>name<Q>:<Q>UvToken<Q>,<Q>description<Q>:<Q>A safe, convenient and efficient decentralized digital asset management community<Q>,<Q>homepage<Q>:<Q>https://www.uvtoken.com<Q>,<Q>chains<Q>:[<Q>polkadot:91b171bb158e2d3848fa23a9f1c25182<Q>,<Q>eip155:1<Q>,<Q>eip155:97<Q>,<Q>eip155:56<Q>,<Q>eip155:65<Q>,<Q>eip155:66<Q>,<Q>eip155:128<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a0057241-cd91-4a53-7175-016b76bfd900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a0057241-cd91-4a53-7175-016b76bfd900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a0057241-cd91-4a53-7175-016b76bfd900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a0057241-cd91-4a53-7175-016b76bfd900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.uvtoken.com<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/hk/app/uvtoken/id1552556395<Q>,<Q>android<Q>:<Q>https://wallet.uvtoken.com/static/download/android/uvtoken.apk<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:null,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>85db431492aa2e8672e93f4ea7acf10c88b97b867b0d373107af63dc4880f041<Q>:{<Q>id<Q>:<Q>85db431492aa2e8672e93f4ea7acf10c88b97b867b0d373107af63dc4880f041<Q>,<Q>name<Q>:<Q>Frontier<Q>,<Q>description<Q>:<Q>Your Front Seat to Crypto and DeFi<Q>,<Q>homepage<Q>:<Q>https://www.frontier.xyz/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:43114<Q>,<Q>eip155:88<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a78c4d48-32c1-4a9d-52f2-ec7ee08ce200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a78c4d48-32c1-4a9d-52f2-ec7ee08ce200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a78c4d48-32c1-4a9d-52f2-ec7ee08ce200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a78c4d48-32c1-4a9d-52f2-ec7ee08ce200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/frontier-defi-wallet/id1482380988<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.frontierwallet&hl=en_IN&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>frontier://<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Frontier<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#CC703C<Q>,<Q>secondary<Q>:<Q>#FFFFFF<Q>}}},<Q>4ff5b6816dd118b8c362939cfb7332f667ff071a1828aa96c760871e1b5634fd<Q>:{<Q>id<Q>:<Q>4ff5b6816dd118b8c362939cfb7332f667ff071a1828aa96c760871e1b5634fd<Q>,<Q>name<Q>:<Q>ZenGo<Q>,<Q>description<Q>:<Q>ZenGo is the first keyless crypto wallet without any private keys, passwords or seed phrases to manage or lose.<Q>,<Q>homepage<Q>:<Q>https://zengo.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d1794d97-ea1f-4966-be42-9f614bb5d800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d1794d97-ea1f-4966-be42-9f614bb5d800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d1794d97-ea1f-4966-be42-9f614bb5d800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d1794d97-ea1f-4966-be42-9f614bb5d800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/app/zengo-crypto-bitcoin-wallet/id1440147115<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.zengo.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q>https://get.zengo.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>ZenGo<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#35C4BA<Q>,<Q>secondary<Q>:null}}},<Q>c587c2601ccfc456cb7d4d9bb34f12f0fd11ad49faeeb4602860e239b5397843<Q>:{<Q>id<Q>:<Q>c587c2601ccfc456cb7d4d9bb34f12f0fd11ad49faeeb4602860e239b5397843<Q>,<Q>name<Q>:<Q>Vision: Crypto Wallet<Q>,<Q>description<Q>:<Q>Vision helps you to keep track of your crypto portfolio - Receive, Send and Trade 100+ Cryptos!<Q>,<Q>homepage<Q>:<Q>https://www.vision-crypto.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>hybrid<Q>,<Q>image_id<Q>:<Q>64ccf07c-1fba-4473-49e8-dc446e5a5000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/64ccf07c-1fba-4473-49e8-dc446e5a5000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/64ccf07c-1fba-4473-49e8-dc446e5a5000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/64ccf07c-1fba-4473-49e8-dc446e5a5000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.vision-crypto.com/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/vision-crypto-bitcoin-wallet/id1500186931<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.eletac.tronwallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://vision-crypto.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Vision<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#0b121c<Q>,<Q>secondary<Q>:null}}},<Q>a18aeec9fab0c08ca41e7bdaae06cac5700bb628ec75c6381bacd9b2df574895<Q>:{<Q>id<Q>:<Q>a18aeec9fab0c08ca41e7bdaae06cac5700bb628ec75c6381bacd9b2df574895<Q>,<Q>name<Q>:<Q>Monarch Wallet<Q>,<Q>description<Q>:<Q>Secure, Decentralized, DAPP Gateway, *NFTs, Buy, Sell, Earn, Swap & *Recurring & Custom Crypto Payments — Over 1 Million Wallets Generated<Q>,<Q>homepage<Q>:<Q>https://monarchwallet.com<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>,<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>,<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>c664d955-8a1e-4460-3917-4cfcf198f000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c664d955-8a1e-4460-3917-4cfcf198f000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c664d955-8a1e-4460-3917-4cfcf198f000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/c664d955-8a1e-4460-3917-4cfcf198f000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/monarch-wallet/id1386397997<Q>,<Q>android<Q>:null,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://monarchwallet.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Monarch<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#F9A900<Q>,<Q>secondary<Q>:<Q>#282629<Q>}}},<Q>e1882224c4c09a84575c533867d434267c46384f5a365b889605d28b061747c4<Q>:{<Q>id<Q>:<Q>e1882224c4c09a84575c533867d434267c46384f5a365b889605d28b061747c4<Q>,<Q>name<Q>:<Q>BeeWallet<Q>,<Q>description<Q>:<Q>Web3 wallet from BeeDAO, your bridge for entering Metaverse.<Q>,<Q>homepage<Q>:<Q>https://www.bee.com/en<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>ae53ba0b-4d01-42f6-53d8-cc568409b700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ae53ba0b-4d01-42f6-53d8-cc568409b700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ae53ba0b-4d01-42f6-53d8-cc568409b700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ae53ba0b-4d01-42f6-53d8-cc568409b700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:null,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=network.bee.app<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://main.apple.bee9527.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>BeeWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffc04d<Q>,<Q>secondary<Q>:<Q>#ffc04d<Q>}}},<Q>af9a6dfff9e63977bbde28fb23518834f08b696fe8bff6dd6827acad1814c6be<Q>:{<Q>id<Q>:<Q>af9a6dfff9e63977bbde28fb23518834f08b696fe8bff6dd6827acad1814c6be<Q>,<Q>name<Q>:<Q>Status<Q>,<Q>description<Q>:<Q>Status is a private messenger, secure crypto wallet, and Ethereum Web3 DApp browser—in short, one powerful communication tool.<Q>,<Q>homepage<Q>:<Q>https://status.im/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:100<Q>,<Q>eip155:4<Q>,<Q>eip155:3<Q>],<Q>versions<Q>:[<Q>1<Q>,<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e131fa98-8c4f-4680-f5b6-6fb77189c900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e131fa98-8c4f-4680-f5b6-6fb77189c900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e131fa98-8c4f-4680-f5b6-6fb77189c900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e131fa98-8c4f-4680-f5b6-6fb77189c900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/status-private-communication/id1178893006<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=im.status.ethereum&hl=en&gl=US<Q>,<Q>mac<Q>:<Q>https://status.im/get/<Q>,<Q>windows<Q>:<Q>https://status.im/get/<Q>,<Q>linux<Q>:<Q>https://status.im/get/<Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Status<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#4360df<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>21b071705a9b9de1646e6a3a0e894d807d0fa4a208e88fc003ee056021f208e1<Q>:{<Q>id<Q>:<Q>21b071705a9b9de1646e6a3a0e894d807d0fa4a208e88fc003ee056021f208e1<Q>,<Q>name<Q>:<Q>ApolloX<Q>,<Q>description<Q>:<Q>ApolloX is a mobile wallet that combines security and anonymity and it allows users to send, receive and store BEP20 (BSC) tokens.<Q>,<Q>homepage<Q>:<Q>https://www.apollox.com/en<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>80ab63a2-1b32-4140-3577-9fbc8ea82e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/80ab63a2-1b32-4140-3577-9fbc8ea82e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/80ab63a2-1b32-4140-3577-9fbc8ea82e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/80ab63a2-1b32-4140-3577-9fbc8ea82e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.apollox.com/en<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/apx-apollox/id1589405398<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.apollox.android<Q>,<Q>mac<Q>:<Q>https://www.apollox.com/en<Q>,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>ApolloX<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:null}}},<Q>c40b9bcef32fa6ce4e0df98be1420628bbc4957646f742380fe618fcb4ab74f1<Q>:{<Q>id<Q>:<Q>c40b9bcef32fa6ce4e0df98be1420628bbc4957646f742380fe618fcb4ab74f1<Q>,<Q>name<Q>:<Q>RiceWallet<Q>,<Q>description<Q>:<Q>Investing in DeFi for Everyone<Q>,<Q>homepage<Q>:<Q>https://ricewallet.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:137<Q>,<Q>eip155:56<Q>,<Q>eip155:43114<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>df94578e-19be-4f00-258f-2470343e7b00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/df94578e-19be-4f00-258f-2470343e7b00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/df94578e-19be-4f00-258f-2470343e7b00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/df94578e-19be-4f00-258f-2470343e7b00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://ricewallet.io<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>ricewallet<Q>,<Q>universal<Q>:<Q>https://ricewallet.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://ricewallet.io<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>RICE Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#3361f4<Q>,<Q>secondary<Q>:null}}},<Q>dc5415b6ea8114db518ae91195b027d690a11a1d2bfdd1a904e93c5cb746380e<Q>:{<Q>id<Q>:<Q>dc5415b6ea8114db518ae91195b027d690a11a1d2bfdd1a904e93c5cb746380e<Q>,<Q>name<Q>:<Q>SimpleHold<Q>,<Q>description<Q>:<Q>SimpleHold is a non-custodial multicurrency wallet that empowers you to receive, send, exchange and store your cryptocurrencies.<Q>,<Q>homepage<Q>:<Q>https://simplehold.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:50<Q>,<Q>eip155:56<Q>,<Q>eip155:61<Q>,<Q>eip155:137<Q>,<Q>eip155:250<Q>,<Q>eip155:361<Q>,<Q>eip155:1666600003<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a9f1ba96-b658-4d13-f71f-226b6389f000<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a9f1ba96-b658-4d13-f71f-226b6389f000/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a9f1ba96-b658-4d13-f71f-226b6389f000/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a9f1ba96-b658-4d13-f71f-226b6389f000/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://simplehold.io/download<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/gb/app/simplehold-crypto-wallet/id1589064973<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.simplehold.app<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>SimpleHold<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#3FBB7D<Q>,<Q>secondary<Q>:<Q>#E9F5EE<Q>}}},<Q>5265dcf66be0553328dbc727414ab329e22f9a480e593bd2e927b279e4ab244d<Q>:{<Q>id<Q>:<Q>5265dcf66be0553328dbc727414ab329e22f9a480e593bd2e927b279e4ab244d<Q>,<Q>name<Q>:<Q>SahalWallet<Q>,<Q>description<Q>:<Q>Non-custodial multi-chain wallet serving as gateway to MRHB Network.<Q>,<Q>homepage<Q>:<Q>https://mrhb.network/<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>,<Q>eip155:97<Q>,<Q>eip155:137<Q>,<Q>eip155:80001<Q>,<Q>eip155:1<Q>,<Q>eip155:4<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d29d6426-b6f2-481b-12d8-7b20ec82af00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d29d6426-b6f2-481b-12d8-7b20ec82af00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d29d6426-b6f2-481b-12d8-7b20ec82af00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d29d6426-b6f2-481b-12d8-7b20ec82af00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/gb/app/sahal-wallet/id1602366920<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=sahal.wallet.app&gl=GB<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:null,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:null}}},<Q>c39d8ee468e50474fdf3a0bd6b981be404d4671e2702a3d633aae95bcbaa032a<Q>:{<Q>id<Q>:<Q>c39d8ee468e50474fdf3a0bd6b981be404d4671e2702a3d633aae95bcbaa032a<Q>,<Q>name<Q>:<Q>XFUN Wallet<Q>,<Q>description<Q>:<Q>XFUN Wallet is a non-custodial wallet that can store, send, and receive FUN, XFUN, other ERC-20 tokens, and BTC.<Q>,<Q>homepage<Q>:<Q>https://xfun.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:137<Q>,<Q>eip155:80001<Q>,<Q>eip155:3<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a665f8f3-09ef-4d17-2bd0-26dca4518400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a665f8f3-09ef-4d17-2bd0-26dca4518400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a665f8f3-09ef-4d17-2bd0-26dca4518400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a665f8f3-09ef-4d17-2bd0-26dca4518400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/app/xfun-wallet/id1612225910<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.xfun.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>xfunwallet://wc<Q>,<Q>universal<Q>:<Q>https://xfun.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>XFUN Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#FFFFFF<Q>,<Q>secondary<Q>:<Q>#FFFFFF<Q>}}},<Q>9034d54985807aaf3d7780f50f155f954daa468fb58d7b14b216fc79d68bbd14<Q>:{<Q>id<Q>:<Q>9034d54985807aaf3d7780f50f155f954daa468fb58d7b14b216fc79d68bbd14<Q>,<Q>name<Q>:<Q>Qubic Wallet<Q>,<Q>description<Q>:<Q>More Than Just A Wallet<Q>,<Q>homepage<Q>:<Q>https://wallet.qubic.app<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:137<Q>,<Q>eip155:80001<Q>,<Q>eip155:4<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>2e9ef302-daae-4807-555f-d4986b0b6700<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2e9ef302-daae-4807-555f-d4986b0b6700/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2e9ef302-daae-4807-555f-d4986b0b6700/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/2e9ef302-daae-4807-555f-d4986b0b6700/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://wallet.qubic.app<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/qubic-%E6%9C%80%E7%B0%A1%E5%96%AE%E4%B8%8A%E6%89%8B%E7%9A%84%E8%99%9B%E6%93%AC%E8%B2%A8%E5%B9%A3%E9%8C%A2%E5%8C%85/id1563987988?itsct=apps_box_link&itscg=30200<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=app.qubic.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>qubic:<Q>,<Q>universal<Q>:<Q>https://wallet.qubic.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Qubic<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>47bb07617af518642f3413a201ec5859faa63acb1dd175ca95085d35d38afb83<Q>:{<Q>id<Q>:<Q>47bb07617af518642f3413a201ec5859faa63acb1dd175ca95085d35d38afb83<Q>,<Q>name<Q>:<Q>KEYRING PRO<Q>,<Q>description<Q>:<Q>KEYRING PRO brings possibilities to reality by offering a simple cross-chain environment, where user can experience multiple chains at once.<Q>,<Q>homepage<Q>:<Q>https://keyring.app/<Q>,<Q>chains<Q>:[<Q>eip155:61<Q>,<Q>eip155:42<Q>,<Q>eip155:88<Q>,<Q>eip155:250<Q>,<Q>eip155:4002<Q>,<Q>eip155:137<Q>,<Q>eip155:80001<Q>,<Q>eip155:10<Q>,<Q>eip155:69<Q>,<Q>eip155:42161<Q>,<Q>eip155:421611<Q>,<Q>eip155:56<Q>,<Q>eip155:97<Q>,<Q>eip155:128<Q>,<Q>eip155:256<Q>,<Q>eip155:1666600000<Q>,<Q>eip155:1666600001<Q>,<Q>eip155:43114<Q>,<Q>eip155:43113<Q>],<Q>versions<Q>:[<Q>1<Q>,<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>dda0f0fb-34e8-4a57-dcea-b008e7d1ff00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dda0f0fb-34e8-4a57-dcea-b008e7d1ff00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dda0f0fb-34e8-4a57-dcea-b008e7d1ff00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/dda0f0fb-34e8-4a57-dcea-b008e7d1ff00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://keyring.app/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/keyring-pro-wallet-management/id1546824976<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=co.bacoor.keyring<Q>,<Q>mac<Q>:<Q>https://keyring.app/<Q>,<Q>windows<Q>:<Q>https://keyring.app/<Q>,<Q>linux<Q>:<Q>https://keyring.app/<Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>keyring:<Q>,<Q>universal<Q>:<Q>https://keyring.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q>keyring:<Q>,<Q>universal<Q>:<Q>https://keyring.app<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>KEYRING PRO<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#00D2C9<Q>,<Q>secondary<Q>:<Q>#FFAA55<Q>}}},<Q>576c90ceaea34f29ff0104837cf2b2e23d201be43be1433feeb18d375430e1fd<Q>:{<Q>id<Q>:<Q>576c90ceaea34f29ff0104837cf2b2e23d201be43be1433feeb18d375430e1fd<Q>,<Q>name<Q>:<Q>PLTwallet<Q>,<Q>description<Q>:<Q>PLTwallet is a wallet for ethereum and PLT<Q>,<Q>homepage<Q>:<Q>https://hashport.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>a5d9dd15-8cef-42de-8bed-09e01a8b0200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a5d9dd15-8cef-42de-8bed-09e01a8b0200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a5d9dd15-8cef-42de-8bed-09e01a8b0200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/a5d9dd15-8cef-42de-8bed-09e01a8b0200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/id1581055631<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.palettechain.wallet&hl=ja&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>pltwallet:<Q>,<Q>universal<Q>:<Q>https://hashport.io/<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>PLTwallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#ffffff<Q>,<Q>secondary<Q>:null}}},<Q>c29c9237e92bc18e141e52aba3aa6d04b1afbe9952a0ab2f96dbd8653645c1df<Q>:{<Q>id<Q>:<Q>c29c9237e92bc18e141e52aba3aa6d04b1afbe9952a0ab2f96dbd8653645c1df<Q>,<Q>name<Q>:<Q>Ancrypto Wallet<Q>,<Q>description<Q>:<Q>Ancrypto Wallet is mnemonics based highly secured mobile wallet to store crypto assets. <Q>,<Q>homepage<Q>:<Q>https://www.antiersolutions.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>polkadot:91b171bb158e2d3848fa23a9f1c25182<Q>,<Q>polkadot:b0a8d493285c2df73290dfb7e61f870f<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d4382329-e288-4d7a-0ac8-3eb0facfb900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d4382329-e288-4d7a-0ac8-3eb0facfb900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d4382329-e288-4d7a-0ac8-3eb0facfb900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d4382329-e288-4d7a-0ac8-3eb0facfb900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/in/app/ancrypto-wallet/id1453657650<Q>,<Q>android<Q>:null,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>ancrypto://app<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Ancrypto Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>540148afe64558bb238cab6c43bd963055ed9248c094eaebff94d7bbb59f9aba<Q>:{<Q>id<Q>:<Q>540148afe64558bb238cab6c43bd963055ed9248c094eaebff94d7bbb59f9aba<Q>,<Q>name<Q>:<Q>TTM Wallet<Q>,<Q>description<Q>:<Q>TTM Wallet is the best app with a friendly user and convenient interface to keep and manage your digital assets.<Q>,<Q>homepage<Q>:<Q>https://ttmbank.com/key-wallet/<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>,<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>82014e92-838b-4e75-e77e-76cdc5539d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/82014e92-838b-4e75-e77e-76cdc5539d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/82014e92-838b-4e75-e77e-76cdc5539d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/82014e92-838b-4e75-e77e-76cdc5539d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/ru/app/ttm-wallet/id1540851562<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.ttmbank.wallet.app&hl=ru&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>ttmwalletapp:<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>TTM Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1BD099<Q>,<Q>secondary<Q>:<Q>#1BD099<Q>}}},<Q>00e39f835988d1bb783b2a0748e18bc6278dec03492d00b0e102a466cd8b3d77<Q>:{<Q>id<Q>:<Q>00e39f835988d1bb783b2a0748e18bc6278dec03492d00b0e102a466cd8b3d77<Q>,<Q>name<Q>:<Q>Zelus<Q>,<Q>description<Q>:<Q>Welcome to the best place to hold and view your NFTs! <Q>,<Q>homepage<Q>:<Q>https://zelus.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>09db4fa8-b905-4300-e254-a2e97c6fed00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/09db4fa8-b905-4300-e254-a2e97c6fed00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/09db4fa8-b905-4300-e254-a2e97c6fed00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/09db4fa8-b905-4300-e254-a2e97c6fed00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/zelus/id1588430343<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.zelus.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>zeluswallet://<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Zelus<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>971e689d0a5be527bac79629b4ee9b925e82208e5168b733496a09c0faed0709<Q>:{<Q>id<Q>:<Q>971e689d0a5be527bac79629b4ee9b925e82208e5168b733496a09c0faed0709<Q>,<Q>name<Q>:<Q>MetaX<Q>,<Q>description<Q>:<Q>One DeFi portal to rule them all<Q>,<Q>homepage<Q>:<Q>https://www.okx.com/metax<Q>,<Q>chains<Q>:[<Q>eip155:66<Q>,<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>45f2f08e-fc0c-4d62-3e63-404e72170500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/45f2f08e-fc0c-4d62-3e63-404e72170500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/45f2f08e-fc0c-4d62-3e63-404e72170500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/45f2f08e-fc0c-4d62-3e63-404e72170500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.okx.com/download<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/okx-buy-bitcoin-eth-crypto/id1327268470<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.okinc.okex.gp<Q>,<Q>mac<Q>:<Q>https://www.okx.com/download<Q>,<Q>windows<Q>:<Q>https://www.okx.com/download<Q>,<Q>linux<Q>:<Q>https://www.okx.com/download<Q>},<Q>mobile<Q>:{<Q>native<Q>:<Q>okex://main<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>MetaX<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q><Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>ce0ea4417b28e4b219283a99dc34cd27afe01e88d4a0cfa41b90c73088949ace<Q>:{<Q>id<Q>:<Q>ce0ea4417b28e4b219283a99dc34cd27afe01e88d4a0cfa41b90c73088949ace<Q>,<Q>name<Q>:<Q>THORWallet<Q>,<Q>description<Q>:<Q>Mobile cross-chain DEX<Q>,<Q>homepage<Q>:<Q>https://thorwallet.org<Q>,<Q>chains<Q>:[<Q>eip155:137<Q>,<Q>eip155:1<Q>,<Q>eip155:43114<Q>,<Q>eip155:1284<Q>,<Q>eip155:3<Q>,<Q>eip155:42161<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>19a02756-462c-4e8a-2d32-af0f9bcf3d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/19a02756-462c-4e8a-2d32-af0f9bcf3d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/19a02756-462c-4e8a-2d32-af0f9bcf3d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/19a02756-462c-4e8a-2d32-af0f9bcf3d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://thorwallet.org<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/ch/app/thorwallet-defi-wallet/id1592064324?l=en<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=defisuisse.thorwallet&hl=de_CH&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>thorwallet:<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>THORWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000000<Q>,<Q>secondary<Q>:<Q>#00CCFF<Q>}}},<Q>7e94e75c90964a69ea375b92214f50c4223dfbfa4913a3733b615444b322f687<Q>:{<Q>id<Q>:<Q>7e94e75c90964a69ea375b92214f50c4223dfbfa4913a3733b615444b322f687<Q>,<Q>name<Q>:<Q>CoinStats<Q>,<Q>description<Q>:<Q>Manage Your Crypto and DeFi Portfolio From One Place<Q>,<Q>homepage<Q>:<Q>https://coinstats.app/<Q>,<Q>chains<Q>:[<Q>eip155:137<Q>,<Q>eip155:56<Q>,<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>hybrid<Q>,<Q>image_id<Q>:<Q>f989ab84-650b-4ad5-c342-77f3334f1b00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f989ab84-650b-4ad5-c342-77f3334f1b00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f989ab84-650b-4ad5-c342-77f3334f1b00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f989ab84-650b-4ad5-c342-77f3334f1b00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://coinstats.app/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/coin-stats-crypto-portfolio/id1247849330<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.coinstats.crypto.portfolio<Q>,<Q>mac<Q>:<Q>https://apps.apple.com/us/app/coin-stats-crypto-tracker/id1498417304?mt=12<Q>,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>coinstats:<Q>,<Q>universal<Q>:<Q>https://coinstats.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>CoinStats<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000000<Q>,<Q>secondary<Q>:<Q>#FF9332<Q>}}},<Q>71e9729d1bbd8ae7f2142a9b0aeb64f142141adc4be6659176e6fe7cf36ddd6c<Q>:{<Q>id<Q>:<Q>71e9729d1bbd8ae7f2142a9b0aeb64f142141adc4be6659176e6fe7cf36ddd6c<Q>,<Q>name<Q>:<Q>Dentacoin Wallet<Q>,<Q>description<Q>:<Q>Securely store, manage and use Dentacoin tokens (DCN) and ETH without any tech knowledge required.<Q>,<Q>homepage<Q>:<Q>https://wallet.dentacoin.com/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:10<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>34910dc0-9f3b-4407-115d-673707602900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/34910dc0-9f3b-4407-115d-673707602900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/34910dc0-9f3b-4407-115d-673707602900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/34910dc0-9f3b-4407-115d-673707602900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://wallet.dentacoin.com/en<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/dentacoin-wallet/id1478732657<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=wallet.dentacoin.com<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>dentacoin-wallet://<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Dentacoin Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#03b5de<Q>,<Q>secondary<Q>:<Q>#152d73<Q>}}},<Q>1c178b393f66b08f6e0decf743ee11c53a36eeb17faaa9e8c20f9ed2d05a96f3<Q>:{<Q>id<Q>:<Q>1c178b393f66b08f6e0decf743ee11c53a36eeb17faaa9e8c20f9ed2d05a96f3<Q>,<Q>name<Q>:<Q>Fundamenta Mobile<Q>,<Q>description<Q>:<Q>Multi-chain wallet and dex interface for your every day crypto needs<Q>,<Q>homepage<Q>:<Q>https://fundamenta.network<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:4<Q>,<Q>eip155:25<Q>,<Q>eip155:56<Q>,<Q>eip155:77<Q>,<Q>eip155:3<Q>,<Q>eip155:5<Q>,<Q>eip155:10<Q>,<Q>eip155:42<Q>,<Q>eip155:100<Q>,<Q>eip155:421611<Q>,<Q>eip155:80001<Q>,<Q>eip155:338<Q>,<Q>eip155:137<Q>,<Q>eip155:97<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>hybrid<Q>,<Q>image_id<Q>:<Q>79797f9e-a6c7-4284-1a1c-88332f11ea00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/79797f9e-a6c7-4284-1a1c-88332f11ea00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/79797f9e-a6c7-4284-1a1c-88332f11ea00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/79797f9e-a6c7-4284-1a1c-88332f11ea00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/fundamenta-mobile/id1613465164<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=network.fundamenta.mobile<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://app.civiport.online<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Fundamenta<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#26a6de<Q>,<Q>secondary<Q>:<Q>#11a65d<Q>}}},<Q>48e53d96460308a1734614b5d4fdf7ea169e6f998e01eb7b4e18014f57904d67<Q>:{<Q>id<Q>:<Q>48e53d96460308a1734614b5d4fdf7ea169e6f998e01eb7b4e18014f57904d67<Q>,<Q>name<Q>:<Q>helix id<Q>,<Q>description<Q>:<Q>Digital Identity Service Provider<Q>,<Q>homepage<Q>:<Q>https://helixid.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:4<Q>,<Q>eip155:80001<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>4083ef71-8389-4682-ded6-0099236d2e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4083ef71-8389-4682-ded6-0099236d2e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4083ef71-8389-4682-ded6-0099236d2e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/4083ef71-8389-4682-ded6-0099236d2e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/de/app/helix-id/id1469238013?l=en<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.io.helix.id&hl=en&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>helix-id://helix-id.com<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>helixid<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#0021FF<Q>,<Q>secondary<Q>:<Q>#FA3C50<Q>}}},<Q>75ca1aafd91026f435803f9a11e8e4278388e189aa30dc93e532244ade262c57<Q>:{<Q>id<Q>:<Q>75ca1aafd91026f435803f9a11e8e4278388e189aa30dc93e532244ade262c57<Q>,<Q>name<Q>:<Q>Talk+<Q>,<Q>description<Q>:<Q>TALK+ provides end-to-end encrypted for high privacy messaging, with multi-cryptocurrency wallet functionality. <Q>,<Q>homepage<Q>:<Q>https://www.talkapp.org/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:4<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d24cdd56-6f55-42da-631b-c25974c36f00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d24cdd56-6f55-42da-631b-c25974c36f00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d24cdd56-6f55-42da-631b-c25974c36f00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d24cdd56-6f55-42da-631b-c25974c36f00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.talkapp.org/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/hk/app/talk-send-crypto-with-friends/id1547227377?l=en<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=org.talkapp&hl=en&gl=US<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>talkapp:<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Talk+<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#FCD86C<Q>,<Q>secondary<Q>:null}}},<Q>c87c562ce7f3a3ff9f4eed5f5a0edbcbd812db5aed4d14c7e6c044d8b6795d84<Q>:{<Q>id<Q>:<Q>c87c562ce7f3a3ff9f4eed5f5a0edbcbd812db5aed4d14c7e6c044d8b6795d84<Q>,<Q>name<Q>:<Q>Opera Crypto Browser<Q>,<Q>description<Q>:<Q>Ethereum Wallet<Q>,<Q>homepage<Q>:<Q>http://opera.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:42<Q>,<Q>eip155:137<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>877fa1a4-304d-4d45-ca8e-f76d1a556f00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/877fa1a4-304d-4d45-ca8e-f76d1a556f00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/877fa1a4-304d-4d45-ca8e-f76d1a556f00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/877fa1a4-304d-4d45-ca8e-f76d1a556f00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/itunes-u/id1604311726?action=write-review<Q>,<Q>android<Q>:null,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>cryptobrowser://<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Crypto Browser<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>23db748bbf7ba1e737921bee04f54d53356e95533e0ed66c39113324873294e7<Q>:{<Q>id<Q>:<Q>23db748bbf7ba1e737921bee04f54d53356e95533e0ed66c39113324873294e7<Q>,<Q>name<Q>:<Q>RealT Wallet<Q>,<Q>description<Q>:<Q>RealT Wallet is a user-friendly, reliable mobile wallet that gives you complete control over your RealTokens and cryptocurrency.<Q>,<Q>homepage<Q>:<Q>https://realt.co<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:5<Q>,<Q>eip155:100<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>bf1f251b-08a5-4b27-ae4a-201a5f698900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bf1f251b-08a5-4b27-ae4a-201a5f698900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bf1f251b-08a5-4b27-ae4a-201a5f698900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/bf1f251b-08a5-4b27-ae4a-201a5f698900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/realt-wallet/id1545585469<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=co.realt.bridge<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:null,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>ed657064daf740734d4a4ae31406cb294a17dc2dbd422ce90a86ed9816f0ded4<Q>:{<Q>id<Q>:<Q>ed657064daf740734d4a4ae31406cb294a17dc2dbd422ce90a86ed9816f0ded4<Q>,<Q>name<Q>:<Q>Nitrogen Wallet<Q>,<Q>description<Q>:<Q>DeFi and GameFi Solana wallet<Q>,<Q>homepage<Q>:<Q>https://nitrogen.org/<Q>,<Q>chains<Q>:[<Q>solana:8E9rvCKLFQia2Y35HXjjpWzj8weVo44K<Q>,<Q>solana:4sGjMW1sUnHzSxGspuhpqLDx6wiyjNtZ<Q>],<Q>versions<Q>:[<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>af185895-cda5-4eaf-e31b-28b6fe4b0800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/af185895-cda5-4eaf-e31b-28b6fe4b0800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/af185895-cda5-4eaf-e31b-28b6fe4b0800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/af185895-cda5-4eaf-e31b-28b6fe4b0800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/nitrogen-wallet/id1595123469<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=org.nitrogen.mobile_wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://nitrogen.org/wc<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Nitrogen<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>11c5487e4d8dd8bf32d4c92222363df8296a27307b2531be1e25770365392ecb<Q>:{<Q>id<Q>:<Q>11c5487e4d8dd8bf32d4c92222363df8296a27307b2531be1e25770365392ecb<Q>,<Q>name<Q>:<Q>Card Wallet<Q>,<Q>description<Q>:<Q>DeFi payments made easy<Q>,<Q>homepage<Q>:<Q>https://cardstack.com/earn-together<Q>,<Q>chains<Q>:[<Q>eip155:100<Q>,<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>325428cf-c212-4d83-a434-7f48902d2c00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/325428cf-c212-4d83-a434-7f48902d2c00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/325428cf-c212-4d83-a434-7f48902d2c00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/325428cf-c212-4d83-a434-7f48902d2c00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://cardstack.com/ios<Q>,<Q>android<Q>:<Q>https://cardstack.com/android<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>cardwallet://<Q>,<Q>universal<Q>:<Q>https://wallet.cardstack.com<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Card<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>76a3d548a08cf402f5c7d021f24fd2881d767084b387a5325df88bc3d4b6f21b<Q>:{<Q>id<Q>:<Q>76a3d548a08cf402f5c7d021f24fd2881d767084b387a5325df88bc3d4b6f21b<Q>,<Q>name<Q>:<Q>LOBSTR Wallet<Q>,<Q>description<Q>:<Q>LOBSTR is a leading platform for managing Stellar Lumens and other assets issued on the Stellar network.<Q>,<Q>homepage<Q>:<Q>https://lobstr.co/<Q>,<Q>chains<Q>:[<Q>stellar:pubnet<Q>],<Q>versions<Q>:[<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>0dafcaab-0852-47f7-85dd-436b86491d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0dafcaab-0852-47f7-85dd-436b86491d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0dafcaab-0852-47f7-85dd-436b86491d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/0dafcaab-0852-47f7-85dd-436b86491d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://lobstr.co/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/lobstr-stellar-wallet/id1404357892<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.lobstr.client<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>lobstr://wc<Q>,<Q>universal<Q>:<Q>https://lobstr.co/uni/wc<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>LOBSTR<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>5dc61e9d57489bccc11306365361614dac3de1d8eab2a9a7877a95970f68712f<Q>:{<Q>id<Q>:<Q>5dc61e9d57489bccc11306365361614dac3de1d8eab2a9a7877a95970f68712f<Q>,<Q>name<Q>:<Q>PayBolt<Q>,<Q>description<Q>:<Q>World’s first Web3 cross-chain crypto payment ecosystem that accepts ALL tokens<Q>,<Q>homepage<Q>:<Q>https://www.paybolt.com<Q>,<Q>chains<Q>:[<Q>eip155:137<Q>,<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>cc8f4e0c-56a8-465a-6cb6-3e9d60846500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cc8f4e0c-56a8-465a-6cb6-3e9d60846500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cc8f4e0c-56a8-465a-6cb6-3e9d60846500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/cc8f4e0c-56a8-465a-6cb6-3e9d60846500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.paybolt.com<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/paybolt-crypto/id1599880290<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.fincrypt.paybolt<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>paybolt://Wallet<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>PayBolt<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#000000<Q>,<Q>secondary<Q>:null}}},<Q>36d8d9c0c7fe2957149ce8e878f3a01a8611521983362d9b651fb6e508325583<Q>:{<Q>id<Q>:<Q>36d8d9c0c7fe2957149ce8e878f3a01a8611521983362d9b651fb6e508325583<Q>,<Q>name<Q>:<Q>CoinCircle<Q>,<Q>description<Q>:<Q>Earn, Pay, Buy, Borrow Crypto<Q>,<Q>homepage<Q>:<Q>https://coincircle.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>eae63a23-c7ba-4f7e-24b3-e6fc69215d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/eae63a23-c7ba-4f7e-24b3-e6fc69215d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/eae63a23-c7ba-4f7e-24b3-e6fc69215d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/eae63a23-c7ba-4f7e-24b3-e6fc69215d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://coincircle.com<Q>,<Q>ios<Q>:<Q>https://coincircle.com/app<Q>,<Q>android<Q>:<Q>https://coincircle.com/app<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q>https://coincircle.com/app/walletconnect<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>CoinCircle<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#2499c4<Q>,<Q>secondary<Q>:<Q>#1c2d5a<Q>}}},<Q>3a9973b9ee638a3aac3e1d001cabe425bf307602a61faee67942fda314736610<Q>:{<Q>id<Q>:<Q>3a9973b9ee638a3aac3e1d001cabe425bf307602a61faee67942fda314736610<Q>,<Q>name<Q>:<Q>Uniblow<Q>,<Q>description<Q>:<Q>A universal blockchain wallet for cryptos<Q>,<Q>homepage<Q>:<Q>https://uniblow.org/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:5<Q>,<Q>eip155:42<Q>,<Q>eip155:56<Q>,<Q>eip155:59<Q>,<Q>eip155:97<Q>,<Q>eip155:137<Q>,<Q>eip155:421611<Q>,<Q>eip155:80001<Q>,<Q>eip155:62320<Q>,<Q>eip155:44787<Q>,<Q>eip155:43114<Q>,<Q>eip155:43113<Q>,<Q>eip155:42220<Q>,<Q>eip155:42161<Q>,<Q>eip155:250<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>3aa86daa-b885-4686-c443-83355e1b3b00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3aa86daa-b885-4686-c443-83355e1b3b00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3aa86daa-b885-4686-c443-83355e1b3b00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3aa86daa-b885-4686-c443-83355e1b3b00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:null,<Q>android<Q>:null,<Q>mac<Q>:<Q>https://uniblow.org/get<Q>,<Q>windows<Q>:<Q>https://uniblow.org/get<Q>,<Q>linux<Q>:<Q>https://uniblow.org/get<Q>},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Uniblow<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#6D57FF<Q>,<Q>secondary<Q>:null}}},<Q>fbea6f68df4e6ce163c144df86da89f24cb244f19b53903e26aea9ab7de6393c<Q>:{<Q>id<Q>:<Q>fbea6f68df4e6ce163c144df86da89f24cb244f19b53903e26aea9ab7de6393c<Q>,<Q>name<Q>:<Q>Klever Wallet<Q>,<Q>description<Q>:<Q>Klever app is a simple, powerful, smart and secure crypto wallet for BTC, ETH, BNB and other crypto assets serving over 2 million users.<Q>,<Q>homepage<Q>:<Q>https://klever.finance/wallet/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:128<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>8f5bbad8-6a14-4b2c-5343-cc1fca6e4d00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8f5bbad8-6a14-4b2c-5343-cc1fca6e4d00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8f5bbad8-6a14-4b2c-5343-cc1fca6e4d00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/8f5bbad8-6a14-4b2c-5343-cc1fca6e4d00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/id1525584688<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=cash.klever.blockchain.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>kleverwallet:<Q>,<Q>universal<Q>:<Q>https://klever.page.link<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Klever<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#DC3F89<Q>,<Q>secondary<Q>:<Q>#903EDD<Q>}}},<Q>ad47668fc1e4327881f1f8b91f563d7a2c1d6e9596cd6b7d97b5791fe76964e9<Q>:{<Q>id<Q>:<Q>ad47668fc1e4327881f1f8b91f563d7a2c1d6e9596cd6b7d97b5791fe76964e9<Q>,<Q>name<Q>:<Q>Mask Network<Q>,<Q>description<Q>:<Q>The portal to the new, open internet<Q>,<Q>homepage<Q>:<Q>https://mask.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:137<Q>,<Q>eip155:42161<Q>,<Q>eip155:100<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>hybrid<Q>,<Q>image_id<Q>:<Q>51fa27fd-8a21-4de0-c084-528e4a37ad00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/51fa27fd-8a21-4de0-c084-528e4a37ad00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/51fa27fd-8a21-4de0-c084-528e4a37ad00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/51fa27fd-8a21-4de0-c084-528e4a37ad00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://mask.io/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/app/id1478382964<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.dimension.maskbook<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Mask<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1c67f2<Q>,<Q>secondary<Q>:<Q>#FFFFFF<Q>}}},<Q>33e181cb6c0f3e313e20b17129f06f4dd9939a01e3a371cdef389d4dcc29258e<Q>:{<Q>id<Q>:<Q>33e181cb6c0f3e313e20b17129f06f4dd9939a01e3a371cdef389d4dcc29258e<Q>,<Q>name<Q>:<Q>Lead Wallet<Q>,<Q>description<Q>:<Q>The world's simplest crypto wallet and DeFi Access<Q>,<Q>homepage<Q>:<Q>https://leadwallet.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:42220<Q>,<Q>polkadot:91b171bb158e2d3848fa23a9f1c25182<Q>,<Q>eip155:137<Q>,<Q>eip155:56<Q>,<Q>stellar:pubnet<Q>,<Q>polkadot:b0a8d493285c2df73290dfb7e61f870f<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>d0407e64-9afa-4c1a-3290-3b925ad8b200<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d0407e64-9afa-4c1a-3290-3b925ad8b200/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d0407e64-9afa-4c1a-3290-3b925ad8b200/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/d0407e64-9afa-4c1a-3290-3b925ad8b200/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/tt/app/lead-wallet/id1578704913<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.leadWallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>leadwallet://wallet-connect/<Q>,<Q>universal<Q>:<Q>https://ws.leadwallet.io/api/api/v1/wallet-connect/<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Lead<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#F90A88<Q>,<Q>secondary<Q>:<Q>#3E2779<Q>}}},<Q>4c0203e76f3163c472073fa9be9843aabdd01a99cd1d0a5a87db5caa71f68898<Q>:{<Q>id<Q>:<Q>4c0203e76f3163c472073fa9be9843aabdd01a99cd1d0a5a87db5caa71f68898<Q>,<Q>name<Q>:<Q>🧑‍🎤 PunkWallet<Q>,<Q>description<Q>:<Q>Forkable burner wallet built with 🏗 scaffold-eth.<Q>,<Q>homepage<Q>:<Q>https://punkwallet.io<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:4<Q>,<Q>eip155:3<Q>,<Q>eip155:42<Q>,<Q>eip155:5<Q>,<Q>eip155:10<Q>,<Q>eip155:137<Q>,<Q>eip155:42161<Q>,<Q>eip155:80001<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>hybrid<Q>,<Q>image_id<Q>:<Q>93f81bf3-a7f1-4620-013e-5959432e3500<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/93f81bf3-a7f1-4620-013e-5959432e3500/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/93f81bf3-a7f1-4620-013e-5959432e3500/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/93f81bf3-a7f1-4620-013e-5959432e3500/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://punkwallet.io<Q>,<Q>ios<Q>:<Q><Q>,<Q>android<Q>:<Q><Q>,<Q>mac<Q>:<Q><Q>,<Q>windows<Q>:<Q><Q>,<Q>linux<Q>:<Q><Q>},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://punkwallet.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://punkwallet.io<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>🧑‍🎤 PunkWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1890ff<Q>,<Q>secondary<Q>:<Q><Q>}}},<Q>0c5bba82e70a2b62405871af809020a077d110d765c0798eb660ad5d3131b328<Q>:{<Q>id<Q>:<Q>0c5bba82e70a2b62405871af809020a077d110d765c0798eb660ad5d3131b328<Q>,<Q>name<Q>:<Q>Edge Wallet<Q>,<Q>description<Q>:<Q> Edge is a powerful and easy to use cryptocurrency wallet that allows users to easily control their own private keys.<Q>,<Q>homepage<Q>:<Q>https://edge.app/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>f601bc29-4298-422f-dbf7-34dac2884f00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f601bc29-4298-422f-dbf7-34dac2884f00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f601bc29-4298-422f-dbf7-34dac2884f00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/f601bc29-4298-422f-dbf7-34dac2884f00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/edge-bitcoin-wallet/id1344400091<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=co.edgesecure.app<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>edge://wc<Q>,<Q>universal<Q>:<Q>https://deep.edge.app/wc<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Edge<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#66EDA8<Q>,<Q>secondary<Q>:<Q>#0D2145<Q>}}},<Q>feb6ff1fb426db18110f5a80c7adbde846d0a7e96b2bc53af4b73aaf32552bea<Q>:{<Q>id<Q>:<Q>feb6ff1fb426db18110f5a80c7adbde846d0a7e96b2bc53af4b73aaf32552bea<Q>,<Q>name<Q>:<Q>Cosmostation<Q>,<Q>description<Q>:<Q>Wallet For Cosmos Ecosystem<Q>,<Q>homepage<Q>:<Q>https://www.cosmostation.io/<Q>,<Q>chains<Q>:[<Q>cosmos:cosmoshub-4<Q>,<Q>cosmos:irishub-1<Q>,<Q>cosmos:kava-4<Q>,<Q>eip155:66<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>ea26c3c8-adb6-4dc4-ee02-35d6eee02800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ea26c3c8-adb6-4dc4-ee02-35d6eee02800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ea26c3c8-adb6-4dc4-ee02-35d6eee02800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/ea26c3c8-adb6-4dc4-ee02-35d6eee02800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/kr/app/cosmostation/id1459830339<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=wannabit.io.cosmostaion<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Cosmostation<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#9c6cff<Q>,<Q>secondary<Q>:<Q>#372758<Q>}}},<Q>7902b32d857b8e1a58a6aab7129b56b40320bdb1c4854d5f2bd0b361f7e76d10<Q>:{<Q>id<Q>:<Q>7902b32d857b8e1a58a6aab7129b56b40320bdb1c4854d5f2bd0b361f7e76d10<Q>,<Q>name<Q>:<Q>LZ Wallet<Q>,<Q>description<Q>:<Q>The ideal wallet for storing and trading your digital assets securely and conveniently.<Q>,<Q>homepage<Q>:<Q>https://lz.finance/home<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e3272444-3876-49d3-2f84-004b818d3800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e3272444-3876-49d3-2f84-004b818d3800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e3272444-3876-49d3-2f84-004b818d3800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e3272444-3876-49d3-2f84-004b818d3800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/app/lz-wallet/id1581578160<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.launchzone.lzwallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>LZ Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>08739356e3fc0efd9498696b7831e8b42b0ad7390af663cd3ba3c30866195b34<Q>:{<Q>id<Q>:<Q>08739356e3fc0efd9498696b7831e8b42b0ad7390af663cd3ba3c30866195b34<Q>,<Q>name<Q>:<Q>BCERTin wallet<Q>,<Q>description<Q>:<Q>The BCERTin wallet comes with a set of amazing tools to manage your business & life with unlimited cloud access<Q>,<Q>homepage<Q>:<Q>https://blockcerts.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:3<Q>,<Q>eip155:4<Q>,<Q>eip155:5<Q>,<Q>eip155:137<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e321346d-5ce7-4e75-371e-e4f0bf923900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e321346d-5ce7-4e75-371e-e4f0bf923900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e321346d-5ce7-4e75-371e-e4f0bf923900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e321346d-5ce7-4e75-371e-e4f0bf923900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:null,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.blockcerts.BlockCerts&hl=en_US&gl<Q>,<Q>mac<Q>:<Q>https://blockcerts.com/download-mac/?type=mac<Q>,<Q>windows<Q>:<Q>https://blockcerts.com/download-windows/?type=windows<Q>,<Q>linux<Q>:<Q>https://blockcerts.com/download-debian/?type=debian<Q>},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>BCERTin wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#76b53b<Q>,<Q>secondary<Q>:<Q>#495849<Q>}}},<Q>19418ecfd44963883e4d6abca1adeb2036f3b5ffb9bee0ec61f267a9641f878b<Q>:{<Q>id<Q>:<Q>19418ecfd44963883e4d6abca1adeb2036f3b5ffb9bee0ec61f267a9641f878b<Q>,<Q>name<Q>:<Q>KryptoGO Wallet<Q>,<Q>description<Q>:<Q>The 1st decentralized DeFi Wallet with full compliance. Unlock DeFi earning in your hand.<Q>,<Q>homepage<Q>:<Q>https://kryptogo.com/wallet<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:137<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>3ccbd966-97e8-45a0-1ceb-6141a8978e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3ccbd966-97e8-45a0-1ceb-6141a8978e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3ccbd966-97e8-45a0-1ceb-6141a8978e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/3ccbd966-97e8-45a0-1ceb-6141a8978e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://kryptogo.com/wallet<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/il/app/kryptogo/id1593830910<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.kryptogo.walletapp<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>https://kryptogo.page.link<Q>,<Q>universal<Q>:<Q>https://kryptogo.page.link<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q>https://kryptogo.com<Q>,<Q>universal<Q>:<Q>https://kryptogo.com<Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>KryptoGO<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#FFC211<Q>,<Q>secondary<Q>:<Q>#001F58<Q>}}},<Q>30edc47c24de2727a86d50ba88c3516db28c0494a7c5f0b127e4329e855c6840<Q>:{<Q>id<Q>:<Q>30edc47c24de2727a86d50ba88c3516db28c0494a7c5f0b127e4329e855c6840<Q>,<Q>name<Q>:<Q>Autonomy<Q>,<Q>description<Q>:<Q>Autonomy is the home for all your NFT art and collectibles — a seamless, customizable way for you and others to experience your collection.<Q>,<Q>homepage<Q>:<Q>https://bitmark.com/autonomy<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:4<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>hybrid<Q>,<Q>image_id<Q>:<Q>672e9061-e0c9-45ec-5d9c-9fd10e83e800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/672e9061-e0c9-45ec-5d9c-9fd10e83e800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/672e9061-e0c9-45ec-5d9c-9fd10e83e800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/672e9061-e0c9-45ec-5d9c-9fd10e83e800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/autonomy-app/id1544022728<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.bitmark.autonomy_client<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>autonomy-wc<Q>,<Q>universal<Q>:<Q>https://au.bitmark.com/apps/wc<Q>},<Q>desktop<Q>:{<Q>native<Q>:<Q><Q>,<Q>universal<Q>:<Q><Q>},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Autonomy<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>91628e2ae2228af2145bfac21093ad7be682810ec16af540a9e017ad6b933a81<Q>:{<Q>id<Q>:<Q>91628e2ae2228af2145bfac21093ad7be682810ec16af540a9e017ad6b933a81<Q>,<Q>name<Q>:<Q>NeftiWallet<Q>,<Q>description<Q>:<Q>The platform's NFT WALLET allows storing of the NFTs securely on the blockchain.<Q>,<Q>homepage<Q>:<Q>https://neftipedia.com/<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>1f812dec-be3d-446c-52f7-a79eb0dd5400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1f812dec-be3d-446c-52f7-a79eb0dd5400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1f812dec-be3d-446c-52f7-a79eb0dd5400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/1f812dec-be3d-446c-52f7-a79eb0dd5400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:null,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.NEFTiPEDiA.mp<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>nefti://nefti.id/asset/<Q>,<Q>universal<Q>:<Q>https://nefti.id/asset/<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>NeftiWallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#37747E<Q>,<Q>secondary<Q>:<Q>#58595B<Q>}}},<Q>746fa59e214ba46cfa688e9540a6b3450b514e89f39bd9c5a00b5a7fdaba8351<Q>:{<Q>id<Q>:<Q>746fa59e214ba46cfa688e9540a6b3450b514e89f39bd9c5a00b5a7fdaba8351<Q>,<Q>name<Q>:<Q>GoldBit<Q>,<Q>description<Q>:<Q>GoldBit 因應數位貨幣而生，作為儲存虛擬幣的冷錢包，支援Bitcoin、Ethereum和TRON三大主鏈的幣種，提供快速且便利的操作，並且擁有市面少有多錢包機制，可以輕鬆且安全存放虛擬貨幣。<Q>,<Q>homepage<Q>:<Q>http://goldbit.io/gbapp.php<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>11974ef1-21ab-4806-a2b1-362c31499900<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/11974ef1-21ab-4806-a2b1-362c31499900/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/11974ef1-21ab-4806-a2b1-362c31499900/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/11974ef1-21ab-4806-a2b1-362c31499900/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/tw/app/goldbit錢包/id1551914030<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=com.goldbitpro.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>goldbit://<Q>,<Q>universal<Q>:<Q><Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>GoldBit<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>114efdbef4ce710081c1507f3dbc51f439c96a342cf33397799cd1c84b01a8c5<Q>:{<Q>id<Q>:<Q>114efdbef4ce710081c1507f3dbc51f439c96a342cf33397799cd1c84b01a8c5<Q>,<Q>name<Q>:<Q>S-ONE Wallet<Q>,<Q>description<Q>:<Q>S-ONE Wallet is a convenient and secure multicoin P2P wallet for S-ONE Finance users.<Q>,<Q>homepage<Q>:<Q>https://s-one.finance/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>649bdb8c-76fc-4e13-58f4-66efe0897400<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/649bdb8c-76fc-4e13-58f4-66efe0897400/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/649bdb8c-76fc-4e13-58f4-66efe0897400/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/649bdb8c-76fc-4e13-58f4-66efe0897400/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q><Q>,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/s-one-wallet/id1564871193<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=sone.finance.wallet<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>sone://<Q>,<Q>universal<Q>:<Q>https://sonewallet.app<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>S-ONE Wallet<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>bb88a220ed4dcd3d717ec19b6ac00a672edf92e97ef7c243d35e25ff56a07301<Q>:{<Q>id<Q>:<Q>bb88a220ed4dcd3d717ec19b6ac00a672edf92e97ef7c243d35e25ff56a07301<Q>,<Q>name<Q>:<Q>Boo!<Q>,<Q>description<Q>:<Q>Boo! stands for BUMooN Operation Orchestrator is a crypto wallet that fully operates under decentralize manner <Q>,<Q>homepage<Q>:<Q>https://bumoon.io<Q>,<Q>chains<Q>:[<Q>eip155:56<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>310e86aa-6ed2-4d36-2af3-c72eb1771e00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/310e86aa-6ed2-4d36-2af3-c72eb1771e00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/310e86aa-6ed2-4d36-2af3-c72eb1771e00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/310e86aa-6ed2-4d36-2af3-c72eb1771e00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://apps.apple.com/us/app/boo/id1593710564<Q>,<Q>android<Q>:<Q>https://play.google.com/store/apps/details?id=io.bumoon.boo<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:<Q>https://boo.bumoon.io<Q>},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Boo!<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#1d629b<Q>,<Q>secondary<Q>:<Q>#0f88a5<Q>}}},<Q>23d57b0a48df6cec411e609ddedaa290dae4a844ea90909ddd33aca794574603<Q>:{<Q>id<Q>:<Q>23d57b0a48df6cec411e609ddedaa290dae4a844ea90909ddd33aca794574603<Q>,<Q>name<Q>:<Q>MyWalliD<Q>,<Q>description<Q>:<Q>MyWalliD lets the user store and manage their own identities assets on the browser local memory or to authenticate on the web3.<Q>,<Q>homepage<Q>:<Q>https://www.wallid.io/<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>e6cff623-9671-4a39-acc7-1c2292d7e100<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e6cff623-9671-4a39-acc7-1c2292d7e100/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e6cff623-9671-4a39-acc7-1c2292d7e100/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/e6cff623-9671-4a39-acc7-1c2292d7e100/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://chrome.google.com/webstore/detail/wallid-wallet/eknlkogikcabnjbjnhmjllabckeapdii<Q>,<Q>ios<Q>:null,<Q>android<Q>:null,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>MyWalliD<Q>,<Q>colors<Q>:{<Q>primary<Q>:<Q>#009FB1<Q>,<Q>secondary<Q>:null}}},<Q>9b9be9e11e15dfc7e6914449c78c345a60a3a5a8ec5855df5517eb76c56b6018<Q>:{<Q>id<Q>:<Q>9b9be9e11e15dfc7e6914449c78c345a60a3a5a8ec5855df5517eb76c56b6018<Q>,<Q>name<Q>:<Q>CeloDance<Q>,<Q>description<Q>:<Q>A Celo mobile wallet that supports the management of celo assets, you can transfer in or out of CELO/cUSD/cEUR, and lock your CELO.<Q>,<Q>homepage<Q>:<Q>http://celo.dance/<Q>,<Q>chains<Q>:[<Q>eip155:42220<Q>],<Q>versions<Q>:[<Q>2<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>47c8ab7b-a66c-4949-f0fe-b0c2c169ee00<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/47c8ab7b-a66c-4949-f0fe-b0c2c169ee00/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/47c8ab7b-a66c-4949-f0fe-b0c2c169ee00/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/47c8ab7b-a66c-4949-f0fe-b0c2c169ee00/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:null,<Q>ios<Q>:<Q>https://www.pgyer.com/JbDW<Q>,<Q>android<Q>:<Q>https://www.pgyer.com/JbDW<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>celo://wallet/dappkit/celodance<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>CeloDance<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}},<Q>22046f35fd2ec3937582426f7e803617101a73bbcbe9374375529cebbfc59c47<Q>:{<Q>id<Q>:<Q>22046f35fd2ec3937582426f7e803617101a73bbcbe9374375529cebbfc59c47<Q>,<Q>name<Q>:<Q>Uvtoken<Q>,<Q>description<Q>:<Q>A safe, convenient and efficient decentralized digital asset management community<Q>,<Q>homepage<Q>:<Q>https://www.uvtoken.com<Q>,<Q>chains<Q>:[<Q>eip155:1<Q>,<Q>eip155:56<Q>,<Q>eip155:66<Q>,<Q>polkadot:91b171bb158e2d3848fa23a9f1c25182<Q>,<Q>polkadot:b0a8d493285c2df73290dfb7e61f870f<Q>],<Q>versions<Q>:[<Q>1<Q>],<Q>app_type<Q>:<Q>wallet<Q>,<Q>image_id<Q>:<Q>52b9a9fc-caff-469e-033b-6d6f14e41800<Q>,<Q>image_url<Q>:{<Q>sm<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/52b9a9fc-caff-469e-033b-6d6f14e41800/sm<Q>,<Q>md<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/52b9a9fc-caff-469e-033b-6d6f14e41800/md<Q>,<Q>lg<Q>:<Q>https://imagedelivery.net/_aTEfDRm7z3tKgu9JhfeKA/52b9a9fc-caff-469e-033b-6d6f14e41800/lg<Q>},<Q>app<Q>:{<Q>browser<Q>:<Q>https://www.uvtoken.com/<Q>,<Q>ios<Q>:<Q>https://apps.apple.com/hk/app/uvtoken/id1552556395<Q>,<Q>android<Q>:<Q>https://wallet.uvtoken.com/static/download/android/uvtoken.apk<Q>,<Q>mac<Q>:null,<Q>windows<Q>:null,<Q>linux<Q>:null},<Q>mobile<Q>:{<Q>native<Q>:<Q>UvToken://<Q>,<Q>universal<Q>:null},<Q>desktop<Q>:{<Q>native<Q>:null,<Q>universal<Q>:null},<Q>metadata<Q>:{<Q>shortName<Q>:<Q>Uvtoken<Q>,<Q>colors<Q>:{<Q>primary<Q>:null,<Q>secondary<Q>:null}}}}";
            
            return wallets.Replace("<Q>", "\"");

        }
    }
}
